using System;
using System.IO;
using System.Xml;
using System.Runtime.Serialization.Formatters.Soap;
//using XMLTupleSpaceShared;


namespace Helpers
{
	/// <summary>
	/// auxiliary methods for type-checking and casting
	/// </summary>
	public class Datatypes
	{
		/// <summary>
		/// Checks if object is of primitive type
		/// </summary>
		/// <param name="obj">
		/// object to check
		/// </param>
		/// <returns>
		/// returns true if object is a primitive datatype, false else
		/// </returns>
		public static bool isPrimitive(Object obj) 
		{
			switch(obj.GetType().ToString()) 
			{
				// all these cases can be handled by ToString()
				case "System.Byte" : return true;
				case "System.SByte" : return true;
				case "System.Int16" : return true;
				case "System.Int32" : return true;
				case "System.Int64" : return true;
				case "System.UInt16" : return true;
				case "System.UInt32" : return true;
				case "System.UInt64" : return true;
				case "System.Single" : return true;
				case "System.Double" : return true;
				case "System.Boolean" : return true;
				case "System.Char" : return true;
				case "System.Decimal" : return true;
				case "System.IntPtr" : return true;
				case "System.UIntPtr" : return true;
				case "System.String" : return true;
				default : return false;
			}
			return false;
		}

		/// <summary>
		/// casts value-strings back to the type specified
		/// </summary>
		/// <param name="type">the type to cast back to</param>
		/// <param name="val">the value</param>
		/// <returns>the actual type with the given calue</returns>
		public static object castToObject(String type, String val) 
		{
			switch( type) 
			{
				case "System.Byte" : return Byte.Parse(val);
				case "System.SByte" : return SByte.Parse(val);
				case "System.Int16" : return Int16.Parse(val);
				case "System.Int32" : return Int32.Parse(val);
				case "System.Int64" : return Int64.Parse(val);
				case "System.UInt16" : return UInt16.Parse(val);
				case "System.UInt32" : return UInt32.Parse(val);
				case "System.UInt64" : return UInt64.Parse(val);
				case "System.Single" : return Single.Parse(val);
				case "System.Double" : return Double.Parse(val);
				case "System.Boolean" : return Boolean.Parse(val);
				case "System.Char" : return Char.Parse(val);
				case "System.Decimal" : return Decimal.Parse(val);
					// ist ein hack, da die Pointer kein Parse() haben
				case "System.IntPtr" : return new IntPtr(Int64.Parse(val));
				case "System.UIntPtr" : return new UIntPtr(UInt64.Parse(val));
				case "System.String" : return val;
					//case "IMatchable": return deserializeObject(n.OuterXml);
				default : return deserializeObject(val);
			}
		}

		/// <summary>
		/// cast an XmlNode to the according type
		/// </summary>
		/// <param name="n">the XmlNode to convert</param>
		/// <returns>the converted object</returns>
		/// 
		public static object castToObject(XmlNode n) 
		{
			string nodeValue = n.InnerXml;
			string nodeType = ((XmlElement) n).GetAttribute("type");
            return castToObject(nodeType, nodeValue);
		}
		/// <summary>
		/// deserialize a string
		/// </summary>
		/// <param name="s">the string to deserialize</param>
		/// <returns>the deserialized object</returns>
		public static object deserializeObject(string s) 
		{
			SoapFormatter soap = new SoapFormatter();
			MemoryStream mstream = new MemoryStream();
			StreamWriter streamw = new StreamWriter(mstream);
			streamw.Write(s);
			streamw.Flush();
			mstream.Flush();
			mstream.Position = 0;
			object ob = soap.Deserialize(mstream);
			return ob;
		}
		/// <summary>
		/// Checks type of argument object
		/// </summary>
		/// <param name="obj">
		/// object to check to
		/// </param>
		/// <returns>
		/// returns true if object is a XMLTuple, false else
		/// </returns>
		public static bool isXMLTuple(Object obj) 
		{
			if(obj.GetType().ToString().Equals("XMLTupleSpaceShared.XMLTuple"))			
			{
				return true;
			} 
			else 
			{
				return false;
			}
		}
		/// <summary>
		/// checks type of argumet object
		/// </summary>
		/// <param name="obj">the object to check</param>
		/// <returns>true, if object is an XMLTemplate, false otherwise</returns>
		public static bool isXMLTemplate(Object obj) 
		{
			if (obj.GetType().BaseType.ToString().Equals("XMLTupleSpaceShared.XMLTemplate"))
			{
				return true;
			} 
			else 
			{
				return false;
			}
		}

		/// <summary>
		/// Checks type of argument object
		/// </summary>
		/// <param name="obj">
		/// object to check to
		/// </param>
		/// <returns>
		/// returns true if object is not a XMLTuple and not a primitive datatype, false else
		/// </returns>
		public static bool isObject(Object obj) 
		{
			return (!isXMLTuple(obj) && !isPrimitive(obj) && !isXMLTemplate(obj));
		}
	}
}
