using System;
using System.Xml;
using System.Net;

namespace Helpers
{
	/// <summary>
	/// This utility-class reads the configuration file and stores the URI of the wellknown-service type (the server) into a URIBuilder. The information is retrieved via provided methods
	/// </summary>
	public class TupleSpaceConfiguraton
	{
		/// <summary>
		/// the UriBuilder instance
		/// </summary>
		private UriBuilder uri;
		/// <summary>
		/// the XmlDoc used to read the xml-file and find the node
		/// </summary>
		private XmlDocument xmldoc;

		/// <summary>
		/// Sinlge constructor takes the path (file location) 
		/// </summary>
		/// <param name="xmlFile">
		/// the location of the xmlfile to read
		/// </param>
		public TupleSpaceConfiguraton(string xmlFile)
		{
			xmldoc = new XmlDocument();
			xmldoc.Load(new XmlTextReader(xmlFile));
			XmlNode node = xmldoc.SelectSingleNode("//wellknown/@url");
			uri = new UriBuilder(node.InnerXml);
		}

		/// <summary>
		/// returns the port the server is listening on
		/// </summary>
		/// <returns>the server port</returns>
		public int getServerPort() 
		{
			return uri.Port;
		}

		/// <summary>
		/// returns the server's name
		/// </summary>
		/// <returns>the server name</returns>
		public string getServerName() 
		{
			return uri.Host;
		}

		/// <summary>
		///  returns the endpoint where the server is stored
		/// </summary>
		/// <returns>the endpoint </returns>
		public string getEndPoint() {
			return uri.Path;
		}
		/// <summary>
		/// gets the full server address with name and port 
		/// </summary>
		/// <returns>the server address</returns>
		public string getFullServerString() 
		{
			return uri.Scheme + "://" + getServerName() + ":" + getServerPort();
		}

		/// <summary>
		/// returns the URI object
		/// </summary>
		/// <returns>the uri</returns>
		public Uri getUri() 
		{
			return uri.Uri;
		}

		/// <summary>
		/// builds a unique id using the host's url and the utility class GUID
		/// </summary>
		/// <returns>the GUID</returns>
		public string createGUID() 
		{
			return getUri().ToString() + ":" + Guid.NewGuid();
		}
	}
}
