using System;
using System.Text;
using Interfaces;

namespace MatchingRelations
{
	/// <summary>
	/// This class implements a very simple match on strings. An expression may contain one "*" at the most. The "*" 
	/// is used as a wildcard for any sequence of characters.
	/// </summary>
	/// 
	[Serializable]
	public class SimpleStringMatch :  IMatchable
	{
		/// <summary>
		/// the expression to match
		/// </summary>
		string expression;
		/// <summary>
		/// Constructor receives the expression to match
		/// </summary>
		/// <param name="matchExpression">the expression to match</param>
		public SimpleStringMatch(string matchExpression)
		{
			expression = matchExpression;
		}

		/// <summary>
		/// if the object is a string, check if the stored expression matches
		/// </summary>
		/// <param name="o">the object to match the expression on</param>
		/// <returns>true, if expresion matches, false otherwise</returns>
		public bool matches(object o) 
		{
			try 
			{
				if(!(o is String)) return false;
				string stringToMatch = (string) o;
				if(expression.Length > stringToMatch.Length+1) return false;
				if(stringToMatch.Equals(expression)) return true;
				return checkWildCards(expression, stringToMatch);
			} 
			catch (InvalidCastException iex) 
			{	
				throw new MatchingException(iex.Message);
			}
		}

		/// <summary>
		/// checks if there are any wildcards match
		/// </summary>
		/// <param name="expression">the stored expression</param>
		/// <param name="stringToMatch">the given string</param>
		/// <returns>true if there is a match, false otherwise</returns>
		private bool checkWildCards(string expression, string stringToMatch) 
		{
			// *yz
			if(expression.StartsWith("*")) 
			{
				if(stringToMatch.EndsWith(expression.Substring(1, expression.Length-1))) 
				{
					return true;
				} 			
			}
			// xy*
			else if(expression.EndsWith("*")) 
			{
				if(stringToMatch.StartsWith(expression.Substring(0, expression.Length-2))) 
				{
					return true;
				} 
			}
			// x*z
			//int index = expression.IndexOf("*");
			else if(expression.IndexOf("*") != -1) 
			{
				int index = expression.IndexOf("*");
				if(stringToMatch.StartsWith(expression.Substring(0, index-1)) 
					&&
				   stringToMatch.EndsWith(expression.Substring(index+1, expression.Length-(index+1))))
				return true;
			}
			return false;
		}
		}
}

