using System;
using System.Xml;
using System.Xml.XPath;
using Interfaces;

namespace MatchingRelations
{
	/// <summary>
	/// Provides an XPath-match method
	/// </summary>
	/// 
	[Serializable]
	public class XPathMatch : IXMLMatchable
	{
		/// <summary>
		/// the XPath - expression
		/// </summary>
		private string xpathExpression = "";
		
		/// <summary>
		/// constructor receives an XPath-expression
		/// </summary>
		/// <param name="xpathExpression"></param>
		public XPathMatch(string xpathExpression)
		{
			this.xpathExpression = xpathExpression;
		}

		/// <summary>
		/// this method evaluates the <c>xpathExpression</c> on the incoming XmlDocument
		/// </summary>
		/// <param name="xmldoc">the XmlDocument</param>
		/// <returns>true, if xpath-expression has a result, false else</returns>
		public bool matches(XmlDocument xmldoc) 
		{   
			
				XmlNodeList nl = xmldoc.SelectNodes(xpathExpression);
				return (nl.Count > 0);
		}
	}
}
