using System;
using Interfaces;
using System.Xml.Xsl;
using System.Xml;
using System.IO;

namespace MatchingRelations
{
	/// <summary>
	/// provides an XSL - match method
	/// </summary>
	public class XSLMatch : IXMLMatchable
	{
		/// <summary>
		/// the XSLT-transformer
		/// </summary>
		XslTransform xslt;
		/// <summary>
		/// the stylesheet location/string representation
		/// </summary>
		string stylesheet = "";
		/// <summary>
		/// takes url, file or string-representation of a stylesheet
		/// </summary>
		/// <param name="stylesheet"></param>
		public XSLMatch(String style)
		{
			this.stylesheet = style;
			xslt = new XslTransform();
			if(isFile(stylesheet) || isURL(stylesheet)) 
			{
				XmlTextReader xmltr = new XmlTextReader(stylesheet);
				XmlUrlResolver xmlresolver = new XmlUrlResolver();
				Console.WriteLine("URL or file");
				xslt.Load(xmltr);
			} 
			else if(isStyleSheet(stylesheet))
			{
				Console.WriteLine("Stylesheet");
				XmlDocument doc = new XmlDocument();
				doc.LoadXml(stylesheet);
				xslt.Load(doc.CreateNavigator());
			}
			Console.WriteLine("XSLMatch");
		}

		/// <summary>
		/// checks if a string is a local file
		/// </summary>
		/// <param name="name">name of stylesheet</param>
		/// <returns>true if file, false otherwise</returns>
		private bool isFile(String name) 
		{
			FileInfo f = new FileInfo(name);
			return f.Exists;			
		}

		/// <summary>
		/// checks if a string is an URL
		/// </summary>
		/// <param name="name">name of stylesheet</param>
		/// <returns>true if URL, false otherwise</returns>
		private bool isURL(String name) 
		{
			try 
			{
				UriBuilder uribuilder = new UriBuilder(name);
				XmlDocument doc = new XmlDocument();			
				doc.Load(name);				
				return true;
			} 
			catch (Exception ex) 
			{
				Console.WriteLine(ex.StackTrace);
				return false;
			}
		}

		/// <summary>
		/// checks if string is a stylesheet and not just a location
		/// </summary>
		/// <param name="name">the stylesheet</param>
		/// <returns>true if a valid stylesheet, false otherwise</returns>
		private bool isStyleSheet(String name) 
		{
			try 
			{
				// ought to be checked against a schema !
				XmlDocument doc = new XmlDocument();
				doc.LoadXml(name);
				return true;
			} 
			catch (Exception ex) 
			{
				Console.WriteLine(ex.StackTrace);
				return false;
			}
		}
/// <summary>
/// using the <c>stylesheet</c> the method transforms the XmlDocument
/// </summary>
/// <param name="o">the XmlDocument</param>
/// <returns>if the resulting XmlDocument is not empty true, else false</returns>
		public bool matches(XmlDocument o) 
		{
			XsltArgumentList xsltargs = new XsltArgumentList();
			MemoryStream mstream = new MemoryStream();
			xslt.Transform(o.CreateNavigator(), xsltargs, mstream);
			if(mstream.GetBuffer().Length < 1)
				return false;
			else
				return true;
		}

	}
}
