using System;
using System.Xml;
using Interfaces;
using System.Collections;
using System.Reflection;

namespace XMLTupleSpaceShared
{
	/// <summary>
	/// A DeepTemplate performs the Linda-matching
	/// </summary>
	public class DeepTemplate : XMLTemplate
	{	
		/// <summary>
		/// creates a DeepTemplate without fields
		/// </summary>
		public DeepTemplate():base(){}
		/// <summary>
		/// stores a fieldTemplate in a DeepTemplate
		/// </summary>
		/// <param name="fieldTemplate">the fieldTemplate</param>
		public DeepTemplate(FieldTemplate fieldTemplate) : base(fieldTemplate){}
		/// <summary>
		/// stores a TupleTemplate in a DeepTemplate
		/// </summary>
		/// <param name="fieldTemplate">the tupleTemplate</param>
		public DeepTemplate(TupleTemplate tupleTemplate): base(tupleTemplate){}
		/// <summary>
		/// stores an <code>int</code> as an XMLField in the DeepTemplate
		/// </summary>
		/// <param name="i">the value of the field</param>
		public DeepTemplate(int i): base(i){}
		/// <summary>
		/// stores a byte as an XMLField in the DeepTemplate
		/// </summary>
		/// <param name="b">the value of the field</param>
		public DeepTemplate(byte b):base(b){}
		/// <summary>
		/// stores a SByte as an XMLField in the DeepTemplate
		/// </summary>
		/// <param name="sb">the value of the field</param>
		public DeepTemplate(SByte sb) : base(sb){}
		/// <summary>
		/// stores a short as an XMLField in the DeepTemplate
		/// </summary>
		/// <param name="s">the value of the field</param>
		public DeepTemplate(short s) : base(s){}
		/// <summary>
		/// stores a long as an XMLField in the DeepTemplate
		/// </summary>
		/// <param name="l">the value of the field</param>
		public DeepTemplate(long l) : base(l){}
		/// <summary>
		/// stores a float as an XMLField in the DeepTemplate
		/// </summary>
		/// <param name="f">the value of the field</param>
		public DeepTemplate(float f) : base(f){}
		/// <summary>
		/// stores a double as an XMLField in the DeepTemplate
		/// </summary>
		/// <param name="d">the value of the field</param>
		public DeepTemplate(double d):base(d){}
		/// <summary>
		/// stores a char as an XMLField in the DeepTemplate
		/// </summary>
		/// <param name="c">the value of the field</param>
		public DeepTemplate(char c):base(c){}
		/// <summary>
		/// stores a string as an XMLField in the DeepTemplate
		/// </summary>
		/// <param name="s">the value of the field</param>
		public DeepTemplate(String s):base(s){}
		/// <summary>
		/// stores a XMLTemplate  in the DeepTemplate
		/// </summary>
		/// <param name="t">the XMLTemplate</param>
		public DeepTemplate(XMLTemplate t):base(t){	}
		/// <summary>
		/// stores an IMatchable object as an XMLField in the DeepTemplate
		/// </summary>
		/// <param name="m">the value of the field</param>
		public DeepTemplate(IMatchable m):base(m){}
		/// <summary>
		/// stores an IXMLMatchable object as an XMLField in the DeepTemplate
		/// </summary>
		/// <param name="m">the value of the field</param>
		public DeepTemplate(IXMLMatchable m):base(m){}
		/// <summary>
		/// stores an object as an XMLField in the DeepTemplate. In further execution the type is differentiated.
		/// </summary>
		/// <param name="o">the value of the field</param>
		public DeepTemplate(Object o):base(o){	}
		/// <summary>
		/// stores an ITuple in the DeepTemplate
		/// </summary>
		/// <param name="t">the ITuple</param>
		public DeepTemplate(ITuple t):base(t) {}
		/// <summary>
		/// Stores an IField in the DeepTemplate
		/// </summary>
		/// <param name="f">the IField</param>
		public DeepTemplate(IField f):base(f){}
		/// <summary>
		/// the match 
		/// </summary>
		/// <param name="xmldoc">the XmlNode to match</param>
		/// <returns>true, if matching, false otherwise</returns>
		public override bool matches(XmlNode xmldoc)
		{
			// traverse xmldocument and choose correct match-method
			return match(this.getXmlDocument().DocumentElement, xmldoc);
		}
		/// <summary>
		/// performs match on two xmlnodes (DocumentElement of XmlDocuments)
		/// </summary>
		/// <param name="templatenode">DocumentElement for template</param>
		/// <param name="tuplenode">DocumentElement of tuple</param>
		/// <returns>true if template and tuple match, false else.</returns>
		public override bool match(XmlNode templatenode, XmlNode tuplenode)
		{
			bool match = true;
			IEnumerator t_enumerator = templatenode.ChildNodes.GetEnumerator();
			IEnumerator n_enumerator = tuplenode.ChildNodes.GetEnumerator();
			//falls anzahl kinder ungleich	
			if(!compareCount(templatenode, tuplenode)) return false;
			// wenn Kinderknoten vorhanden
			while(n_enumerator.MoveNext() && t_enumerator.MoveNext())		
			{	
				string elements_match = elementNameMatch((XmlNode)t_enumerator.Current, (XmlNode) n_enumerator.Current);	
			 if(elements_match.Equals("Tuple"))
				{
					// deep matching
					match = match && deepmatch((XmlNode) t_enumerator.Current, (XmlNode)n_enumerator.Current);
					if(!match) break;
				}
				else if(templatenode.Name.Equals("Template"))
				{
					string type = ((XmlElement)templatenode).GetAttribute("type");
					string assembly  = type.Substring(0, type.LastIndexOf(".") );
				
					try
					{
						object o  = AppDomain.CurrentDomain.CreateInstanceAndUnwrap(assembly, type,true,
							BindingFlags.Default,
							null,
							null,
							null,
							null,
							null
							);
						if(!this.GetType().Equals(o.GetType()))
						{
							match = match && ((XMLTemplate)o).match(templatenode, tuplenode);
						}
					else
					{
						match = match && deepmatch((XmlNode) t_enumerator.Current, (XmlNode)n_enumerator.Current);
						if(!match) break;
					}
					}
					catch(TypeLoadException e)
					{
						Console.WriteLine(e.StackTrace);
					}
					if(!match) break;
				}
				else if(templatenode.Name.Equals("FieldTemplate"))
				{
					match = match && fieldTemplatematch((XmlNode) t_enumerator.Current, (XmlNode)n_enumerator.Current);
					if(!match) break;
				}
				else if(templatenode.Name.Equals("TupleTemplate"))
				{
					match = match && tupleTemplatematch((XmlNode) t_enumerator.Current, (XmlNode)n_enumerator.Current);
					if(!match) break;
				}
				else if(elements_match.Equals("Field"))
				{
					match = match && matchFields((XmlNode) t_enumerator.Current, (XmlNode) n_enumerator.Current);
					if(!match) break;
				}
			}
			return match;
		}
		
		
		/// <summary>
		/// performs a breadth-first traversal on two dom-trees and matches all its elements
		/// </summary>
		/// <param name="fst">current node of the first dom-tree</param>
		/// <param name="snd">current node of the second dom-tree</param>
		/// <returns>true if nodes match and recursively checks childnodes. else returns false and aborts match.</returns>
		private bool deepmatch(XmlNode fst, XmlNode snd) 
		{
			bool match = true;
			IEnumerator fst_enumerator = fst.ChildNodes.GetEnumerator();
			IEnumerator snd_enumerator = snd.ChildNodes.GetEnumerator();
			while(fst_enumerator.MoveNext() && snd_enumerator.MoveNext())		// wenn Kinderknoten vorhanden
			{	
				string names_match = elementNameMatch((XmlNode)fst_enumerator.Current,(XmlNode) snd_enumerator.Current);	
				 if(names_match.Equals("Tuple"))
				{
					 if(!compareCount(fst, snd)) return false;
					match = match && deepmatch((XmlNode)fst_enumerator.Current,(XmlNode) snd_enumerator.Current);
					if (!match) return false;
				}
				else if(names_match.Equals("Field"))
				{
					match = match && matchFields((XmlNode)fst_enumerator.Current,(XmlNode) snd_enumerator.Current);
					if (!match) return false;
				}
				else if(((XmlNode) fst_enumerator.Current).Name.Equals("Template"))
				{
					 if(!compareCount(fst, snd)) return false;
					string type = ((XmlElement)fst).GetAttribute("type");
					string assembly  = type.Substring(0, type.LastIndexOf(".") );
				
					try
					{
						object o  = AppDomain.CurrentDomain.CreateInstanceAndUnwrap(assembly, type,true,
							BindingFlags.Default,
							null,
							null,
							null,
							null,
							null
							);
						if(!this.GetType().Equals(o.GetType()))
						{
							match = match && ((XMLTemplate)o).match(fst, snd);
						}
						else
						{
							match = match && deepmatch((XmlNode)fst_enumerator.Current,(XmlNode) snd_enumerator.Current);
							if(!match) break;
						}
					}
					catch(TypeLoadException e)
					{
						Console.WriteLine(e.StackTrace);
					}	
					if(!match) break;						
				}
				else
				{
				}
			}
			return match;
		}		


		/// <summary>
		/// a flat match means equality of fields in same position and  equality of field- and tuplecount of tuples in same position.
		/// this method checks if fieldcount and tuplecount of both tuples is equal. matching of the fields of this tuple is performed in the calling method.
		/// </summary>
		/// <param name="fst">first xmlnode representing a tuple</param>
		/// <param name="snd">second xmlnode representing a tuple</param>
		/// <returns>true if fieldcount and tuplecount match and false else.</returns>
		public bool flatmatch(XmlNode fst, XmlNode snd)
		{
			return compareCount(fst,snd);
		}
	}
}
