using System;
using System.Xml;
using Interfaces;
using System.Collections;
using System.Reflection;

namespace XMLTupleSpaceShared
{
	/// <summary>
	/// Summary description for DeepXmlTemplate.
	/// </summary>
	//[Serializable]
	public class DeepXMLTemplate : XMLTemplate
	{
		
		
		/// <summary> 
		/// On construction the class creates an XmlDocument for further use
		/// </summary>
		public DeepXMLTemplate():base()
		{
		}
		public DeepXMLTemplate(int i): base(i)
		{
		}
		public DeepXMLTemplate(byte b):base(b)
		{
		}
		public DeepXMLTemplate(SByte sb) : base(sb)
		{
		}
		public DeepXMLTemplate(short s) : base(s)
		{
		}
		public DeepXMLTemplate(long l) : base(l)
		{
		}
		public DeepXMLTemplate(float f) : base(f)
		{
		}
		public DeepXMLTemplate(double d):base(d)
		{
		}
		public DeepXMLTemplate(char c):base(c)
		{
		}
		public DeepXMLTemplate(String s):base(s)
		{
		}
		public DeepXMLTemplate(XMLTemplate t):base(t)
		{			
		}
		public DeepXMLTemplate(IMatchable m):base(m)
		{		
		}		
		public DeepXMLTemplate(IXMLMatchable m):base(m)
		{
		}
		/// <summary>
		/// Stores an object in the tuple. In further execution the type is differentiated.
		/// </summary>
		/// <param name="o"></param>
		public DeepXMLTemplate(Object o):base(o)
		{
			
		}

		public DeepXMLTemplate(ITuple t):base(t)
		{			
		}
		/// <summary>
		/// Stores a field in the tuple
		/// </summary>
		/// <param name="f"></param>
		public DeepXMLTemplate(IField f):base(f)
		{
			
		}

		public override bool matches(XmlNode xmldoc)
		{
			// traverse xmldocument and choose correct match-method
			return match(this.getXmlDocument().DocumentElement, xmldoc);
			//return true;
		}
		/// <summary>
		/// performs match on two xmlnodes (DocumentElement of XmlDocuments)
		/// </summary>
		/// <param name="templatenode">DocumentElement for template</param>
		/// <param name="tuplenode">DocumentElement of tuple</param>
		/// <param name="deep">switch for deep match or flat match</param>
		/// <returns>true if template and tuple match, false else.</returns>
		public override bool match(XmlNode templatenode, XmlNode tuplenode)
		{
			//templatenode = templatenode.FirstChild;
			TimeSpan beginTime = new TimeSpan(DateTime.Now.Ticks);
			logger.Debug("First Node : " + templatenode.OuterXml);
			logger.Debug("Second Node : " + tuplenode.OuterXml);
			logger.Debug("match");
			bool match = true;
			IEnumerator t_enumerator = templatenode.ChildNodes.GetEnumerator();
			IEnumerator n_enumerator = tuplenode.ChildNodes.GetEnumerator();
			//falls anzahl kinder ungleich	
			if(!compareCount(templatenode, tuplenode)) return false;
			// wenn Kinderknoten vorhanden
			logger.Debug("in foreach");		
			while(n_enumerator.MoveNext() && t_enumerator.MoveNext())		
			{	
				logger.Debug("in while");
				string elements_match = elementNameMatch((XmlNode)t_enumerator.Current, (XmlNode) n_enumerator.Current);	
				// stimmt nicht berein, nchstes tuple aus potenziell versuchen zu matchen
				//				Console.WriteLine("elements_match : " + elements_match);
				if(elements_match.Equals(UNEQUAL)) 
				{
					logger.Debug("not equal");
					match = false;
					break;						
				} 				
				else if(elements_match.Equals("Tuple"))
				{
					logger.Debug("a tuple");
					// tiefes matching
					logger.Debug("calling deepmatch");
					match = match && deepmatch((XmlNode) n_enumerator.Current, (XmlNode)t_enumerator.Current);
					if(!match) break;
				}
				else if(templatenode.Name.Equals("Template"))
				{
					logger.Debug("template gefunden____________");
					string type = ((XmlElement)templatenode).GetAttribute("type");
					string assembly  = type.Substring(0, type.LastIndexOf(".") );
				
					try
					{
						object o  = AppDomain.CurrentDomain.CreateInstanceAndUnwrap(assembly, type,true,
							BindingFlags.Default,
							null,
							null,
							null,
							null,
							null
							);
						if(!this.GetType().Equals(o.GetType()))
						{
							match = match && ((XMLTemplate)o).match(templatenode, tuplenode);
						}
					}
					catch(TypeLoadException e)
					{
						Console.WriteLine(e.StackTrace);
					}
					
					if(!match) break;
						
				}
				else if(elements_match.Equals("Field"))
				{
					//					Console.WriteLine("a field, calling matchfield...");
					match = match && matchFields((XmlNode) t_enumerator.Current, (XmlNode) n_enumerator.Current);
					if(!match) break;
				}
			}
			TimeSpan elapsedTime = new TimeSpan(DateTime.Now.Ticks).Subtract(beginTime);
			
			logger.Info("TIME from match______="+elapsedTime.Minutes+":"+elapsedTime.Seconds+":"+elapsedTime.Milliseconds);
			logger.Info("match is_______"+match.ToString());
			return match;
		}
		
		
		/// <summary>
		/// performs a breadth-first traversal on two dom-trees and matches all its elements
		/// </summary>
		/// <param name="fst">current node of the first dom-tree</param>
		/// <param name="snd">current node of the second dom-tree</param>
		/// <returns>true if nodes match and recursively checks childnodes. else returns false and aborts match.</returns>
		private bool deepmatch(XmlNode fst, XmlNode snd) 
		{
			logger.Debug("deepmatch");
			if(!compareCount(fst, snd)) return false;
			bool match = true;
			IEnumerator fst_enumerator = fst.ChildNodes.GetEnumerator();
			IEnumerator snd_enumerator = snd.ChildNodes.GetEnumerator();
			while(fst_enumerator.MoveNext() && snd_enumerator.MoveNext())		// wenn Kinderknoten vorhanden
			{	
				string types_match = typematch((XmlNode)fst_enumerator.Current,(XmlNode) snd_enumerator.Current);	
				if(types_match.Equals(UNEQUAL))   return false; // matcht nicht
				
				else if(types_match.Equals("Tuple"))
				{
					match = match && deepmatch((XmlNode)fst_enumerator.Current,(XmlNode) snd_enumerator.Current);
					if (!match) return false;
				}
				else if(types_match.Equals("Field"))
				{
					match = match && matchFields((XmlNode)fst_enumerator.Current,(XmlNode) snd_enumerator.Current);
					if (!match) return false;
				}
			}
			return match;
		}		


		/// <summary>
		/// a flat match means equality of fields in same position and  equality of field- and tuplecount of tuples in same position.
		/// this method checks if fieldcount and tuplecount of both tuples is equal. matching of the fields of this tuple is performed in the calling method.
		/// </summary>
		/// <param name="fst">first xmlnode representing a tuple</param>
		/// <param name="snd">second xmlnode representing a tuple</param>
		/// <returns>true if fieldcount and tuplecount match and false else.</returns>
		public bool flatmatch(XmlNode fst, XmlNode snd)
		{
			logger.Debug("flatmatch");
			return compareCount(fst,snd);
		}
	}
}
