using System;
using System.Xml;
using System.Text;
using System.Runtime.Serialization;

namespace XMLTupleSpaceShared
{
	/// <summary>
	/// This class extends the standard XmlDocument class by implementing the <c>ISerialiable</c> interface.
	/// </summary>
	[Serializable]
	public class ExtendedXmlDocument : XmlDocument, ISerializable
	{
		/// <summary>
		/// use base-constructor
		/// </summary>
		public ExtendedXmlDocument() : base()
		{			
		}   
		
		/// <summary>
		/// additional constructor to create intialize an XmlDocument with the given string
		/// </summary>
		/// <param name="xmlrep">the xml string</param>
		public ExtendedXmlDocument(string xmlrep) 
		{
			this.LoadXml(xmlrep);
		}

		/// <summary>
		/// Writes the Xml-contents into a file
		/// </summary>
		/// <param name="name">the fileName</param>
		public void writeToFile(String name) 
		{
			XmlTextWriter xmltr = new XmlTextWriter(name, System.Text.Encoding.UTF8);
			this.Save(xmltr);
			xmltr.Flush();
			xmltr.Close();
		}

		/// <summary>
		/// implements ISerializable - method
		/// </summary>
		/// <param name="info">the Serialisationinfo</param>
		/// <param name="ctxt">the StreamingContext</param>
		public ExtendedXmlDocument(SerializationInfo info, StreamingContext ctxt)
		{
			string doc = (string)info.GetValue("StringRepresentation", typeof(string));			
			this.LoadXml(doc);
		}

		/// <summary>
		/// implements ISerializable - method
		/// </summary>
		/// <param name="info">the Serialisationinfo</param>
		/// <param name="ctxt">the StreamingContext</param>
		public void GetObjectData(SerializationInfo info, StreamingContext ctxt)
		{
			string xmldocument = this.OuterXml;
			info.AddValue("StringRepresentation",  xmldocument);
		}
	}
}
