using System;
using System.Xml;
using Interfaces;

namespace XMLTupleSpaceShared
{
	/// <summary>
	/// the sum of the number of fields and tuples has to be equal for successful matching
	/// </summary>
	public class FlatSizeTemplate : XMLTemplate
	{
		/// <summary>
		/// creates a FlatSizeTemplate without fields
		/// </summary>
		public FlatSizeTemplate():base(){}
		/// <summary>
		/// stores a fieldTemplate in a FlatSizeTemplate
		/// </summary>
		/// <param name="fieldTemplate">the fieldTemplate</param>
		public FlatSizeTemplate(FieldTemplate fieldTemplate) : base(fieldTemplate){}
		/// <summary>
		/// stores a TupleTemplate in a FlatSizeTemplate
		/// </summary>
		/// <param name="fieldTemplate">the tupleTemplate</param>
		public FlatSizeTemplate(TupleTemplate tupleTemplate): base(tupleTemplate){}
		/// <summary>
		/// stores an int as an XMLField in the FlatSizeTemplate
		/// </summary>
		/// <param name="i">the value of the field</param>
		public FlatSizeTemplate(int i): base(i){}
		/// <summary>
		/// stores a byte as an XMLField in the FlatSizeTemplate
		/// </summary>
		/// <param name="b">the value of the field</param>
		public FlatSizeTemplate(byte b):base(b){}
		/// <summary>
		/// stores a SByte as an XMLField in the FlatSizeTemplate
		/// </summary>
		/// <param name="sb">the value of the field</param>
		public FlatSizeTemplate(SByte sb) : base(sb){}
		/// <summary>
		/// stores a short as an XMLField in the FlatSizeTemplate
		/// </summary>
		/// <param name="s">the value of the field</param>
		public FlatSizeTemplate(short s) : base(s){}
		/// <summary>
		/// stores a long as an XMLField in the FlatSizeTemplate
		/// </summary>
		/// <param name="l">the value of the field</param>
		public FlatSizeTemplate(long l) : base(l){}
		/// <summary>
		/// stores a float as an XMLField in the FlatSizeTemplate
		/// </summary>
		/// <param name="f">the value of the field</param>
		public FlatSizeTemplate(float f) : base(f){}
		/// <summary>
		/// stores a double as an XMLField in the FlatSizeTemplate
		/// </summary>
		/// <param name="d">the value of the field</param>
		public FlatSizeTemplate(double d):base(d){}
		/// <summary>
		/// stores a char as an XMLField in the FlatSizeTemplate
		/// </summary>
		/// <param name="c">the value of the field</param>
		public FlatSizeTemplate(char c):base(c){}
		/// <summary>
		/// stores a string as an XMLField in the FlatSizeTemplate
		/// </summary>
		/// <param name="s">the value of the field</param>
		public FlatSizeTemplate(String s):base(s){}
		/// <summary>
		/// stores a XMLTemplate  in the FlatSizeTemplate
		/// </summary>
		/// <param name="t">the XMLTemplate</param>
		public FlatSizeTemplate(XMLTemplate t):base(t){	}
		/// <summary>
		/// stores an IMatchable object as an XMLField in the FlatSizeTemplate
		/// </summary>
		/// <param name="m">the value of the field</param>
		public FlatSizeTemplate(IMatchable m):base(m){}
		/// <summary>
		/// stores an IXMLMatchable object as an XMLField in the FlatSizeTemplate
		/// </summary>
		/// <param name="m">the value of the field</param>
		public FlatSizeTemplate(IXMLMatchable m):base(m){}
		/// <summary>
		/// stores an object as an XMLField in the FlatSizeTemplate. In further execution the type is differentiated.
		/// </summary>
		/// <param name="o">the value of the field</param>
		public FlatSizeTemplate(Object o):base(o){	}
		/// <summary>
		/// stores an ITuple in the FlatSizeTemplate
		/// </summary>
		/// <param name="t">the ITuple</param>
		public FlatSizeTemplate(ITuple t):base(t) {}
		/// <summary>
		/// Stores an IField in the FlatSizeTemplate
		/// </summary>
		/// <param name="f">the IField</param>
		public FlatSizeTemplate(IField f):base(f){}
		/// <summary>
		/// the match 
		/// </summary>
		/// <param name="xmldoc">the XmlNode to match</param>
		/// <returns>true, if matching, false otherwise</returns>
		public override bool matches(XmlNode xmldoc) 
		{
			return match(this.getXmlDocument().DocumentElement, xmldoc);
		}
		/// <summary>
		/// the match 
		/// </summary>
		/// <param name="templatenode">the templatenode to match</param>
		/// <param name="tuplenode">the tuplenode to match</param>
		/// <returns>true, if matching, false otherwise</returns>
		public override bool match(XmlNode templatenode, XmlNode tuplenode) 
		{
			int tuplecounttemplate = gettuplecount(templatenode);
			int fieldcounttemplate = getfieldcount(templatenode);
			int tuplecounttuple = gettuplecount(tuplenode);
			int fieldcounttuple = getfieldcount(tuplenode);
			int templatecount=tuplecounttemplate+fieldcounttemplate;
			int tuplecount = tuplecounttuple + fieldcounttuple;
			if (templatecount==tuplecount) return true;
			return false;
		}
	}
}