using System;
using System.Xml;
using Interfaces;
using System.Collections;

namespace XMLTupleSpaceShared
{
	/// <summary>
	/// the sequences of fields and tuples have to be equal for successful matching
	/// </summary>
	public class FlatTemplate : XMLTemplate
	{
		/// <summary>
		/// creates a FlatTemplate without fields
		/// </summary>
		public FlatTemplate():base(){}
		/// <summary>
		/// stores a fieldTemplate in a FlatTemplate
		/// </summary>
		/// <param name="fieldTemplate">the fieldTemplate</param>
		public FlatTemplate(FieldTemplate fieldTemplate) : base(fieldTemplate){}
		/// <summary>
		/// stores a TupleTemplate in a FlatTemplate
		/// </summary>
		/// <param name="fieldTemplate">the tupleTemplate</param>
		public FlatTemplate(TupleTemplate tupleTemplate): base(tupleTemplate){}
		/// <summary>
		/// stores an int as an XMLField in the FlatTemplate
		/// </summary>
		/// <param name="i">the value of the field</param>
		public FlatTemplate(int i): base(i){}
		/// <summary>
		/// stores a byte as an XMLField in the FlatTemplate
		/// </summary>
		/// <param name="b">the value of the field</param>
		public FlatTemplate(byte b):base(b){}
		/// <summary>
		/// stores a SByte as an XMLField in the FlatTemplate
		/// </summary>
		/// <param name="sb">the value of the field</param>
		public FlatTemplate(SByte sb) : base(sb){}
		/// <summary>
		/// stores a short as an XMLField in the FlatTemplate
		/// </summary>
		/// <param name="s">the value of the field</param>
		public FlatTemplate(short s) : base(s){}
		/// <summary>
		/// stores a long as an XMLField in the FlatTemplate
		/// </summary>
		/// <param name="l">the value of the field</param>
		public FlatTemplate(long l) : base(l){}
		/// <summary>
		/// stores a float as an XMLField in the FlatTemplate
		/// </summary>
		/// <param name="f">the value of the field</param>
		public FlatTemplate(float f) : base(f){}
		/// <summary>
		/// stores a double as an XMLField in the FlatTemplate
		/// </summary>
		/// <param name="d">the value of the field</param>
		public FlatTemplate(double d):base(d){}
		/// <summary>
		/// stores a char as an XMLField in the FlatTemplate
		/// </summary>
		/// <param name="c">the value of the field</param>
		public FlatTemplate(char c):base(c){}
		/// <summary>
		/// stores a string as an XMLField in the FlatTemplate
		/// </summary>
		/// <param name="s">the value of the field</param>
		public FlatTemplate(String s):base(s){}
		/// <summary>
		/// stores a XMLTemplate  in the FlatTemplate
		/// </summary>
		/// <param name="t">the XMLTemplate</param>
		public FlatTemplate(XMLTemplate t):base(t){	}
		/// <summary>
		/// stores an IMatchable object as an XMLField in the FlatTemplate
		/// </summary>
		/// <param name="m">the value of the field</param>
		public FlatTemplate(IMatchable m):base(m){}
		/// <summary>
		/// stores an IXMLMatchable object as an XMLField in the FlatTemplate
		/// </summary>
		/// <param name="m">the value of the field</param>
		public FlatTemplate(IXMLMatchable m):base(m){}
		/// <summary>
		/// stores an object as an XMLField in the FlatTemplate. In further execution the type is differentiated.
		/// </summary>
		/// <param name="o">the value of the field</param>
		public FlatTemplate(Object o):base(o){	}
		/// <summary>
		/// stores an ITuple in the FlatTemplate
		/// </summary>
		/// <param name="t">the ITuple</param>
		public FlatTemplate(ITuple t):base(t) {}
		/// <summary>
		/// Stores an IField in the FlatTemplate
		/// </summary>
		/// <param name="f">the IField</param>
		public FlatTemplate(IField f):base(f){}
		/// <summary>
		/// the match 
		/// </summary>
		/// <param name="xmldoc">the XmlNode to match</param>
		/// <returns>true, if matching, false otherwise</returns>
		public override bool matches(XmlNode xmldoc) 
		{
			return match(this.getXmlDocument().DocumentElement, xmldoc);
		}
		/// <summary>
		/// the match 
		/// </summary>
		/// <param name="templatenode">the templatenode to match</param>
		/// <param name="tuplenode">the tuplenode to match</param>
		/// <returns>true, if matching, false otherwise</returns>
		public override bool match(XmlNode templatenode, XmlNode tuplenode) 
		{
			bool match = true;
			IEnumerator t_enumerator = templatenode.ChildNodes.GetEnumerator();
			IEnumerator n_enumerator = tuplenode.ChildNodes.GetEnumerator();
			//falls anzahl kinder ungleich	
			if(!compareCount(templatenode, tuplenode)) return false;
			// wenn Kinderknoten vorhanden
			while(n_enumerator.MoveNext() && t_enumerator.MoveNext())		
			{	
				string elements_match = elementNameMatch((XmlNode)t_enumerator.Current, (XmlNode) n_enumerator.Current);
				if(elements_match.Equals("Field")) 
				{
					match = match && true;
					if(!match) break;
				}
				else if(elements_match.Equals("Tuple"))
				{
					match = match && true;
					if(!match) break;
				}
				else if(((XmlNode) t_enumerator.Current).Name.Equals("Template"))
				{
					match = match && tupleTemplatematch((XmlNode) n_enumerator.Current, (XmlNode)t_enumerator.Current);
					if(!match) break;
				}
				else if(((XmlNode) t_enumerator.Current).Name.Equals("TupleTemplate"))
				{
					match = match && tupleTemplatematch((XmlNode) n_enumerator.Current, (XmlNode)t_enumerator.Current);
					if(!match) break;
				}
				else if(((XmlNode) t_enumerator.Current).Name.Equals("FieldTemplate"))
				{
					match = match && fieldTemplatematch((XmlNode) n_enumerator.Current, (XmlNode)t_enumerator.Current);
					if(!match) break;
				}

			}
			return match;
		}
	}
}
