using System;
using System.Xml;
using Interfaces;

namespace XMLTupleSpaceShared
{
	/// <summary>
	/// the number of fields and the number of tuples have each to be equal for successful matching
	/// </summary>
	public class FlatTypeTemplate : XMLTemplate
	{
		/// <summary>
		/// creates a FlatTypeTemplate without fields
		/// </summary>
		public FlatTypeTemplate():base(){}
		/// <summary>
		/// stores a fieldTemplate in a FlatTypeTemplate
		/// </summary>
		/// <param name="fieldTemplate">the fieldTemplate</param>
		public FlatTypeTemplate(FieldTemplate fieldTemplate) : base(fieldTemplate){}
		/// <summary>
		/// stores a TupleTemplate in a FlatTypeTemplate
		/// </summary>
		/// <param name="fieldTemplate">the tupleTemplate</param>
		public FlatTypeTemplate(TupleTemplate tupleTemplate): base(tupleTemplate){}
		/// <summary>
		/// stores an int as an XMLField in the FlatTypeTemplate
		/// </summary>
		/// <param name="i">the value of the field</param>
		public FlatTypeTemplate(int i): base(i){}
		/// <summary>
		/// stores a byte as an XMLField in the FlatTypeTemplate
		/// </summary>
		/// <param name="b">the value of the field</param>
		public FlatTypeTemplate(byte b):base(b){}
		/// <summary>
		/// stores a SByte as an XMLField in the FlatTypeTemplate
		/// </summary>
		/// <param name="sb">the value of the field</param>
		public FlatTypeTemplate(SByte sb) : base(sb){}
		/// <summary>
		/// stores a short as an XMLField in the FlatTypeTemplate
		/// </summary>
		/// <param name="s">the value of the field</param>
		public FlatTypeTemplate(short s) : base(s){}
		/// <summary>
		/// stores a long as an XMLField in the FlatTypeTemplate
		/// </summary>
		/// <param name="l">the value of the field</param>
		public FlatTypeTemplate(long l) : base(l){}
		/// <summary>
		/// stores a float as an XMLField in the FlatTypeTemplate
		/// </summary>
		/// <param name="f">the value of the field</param>
		public FlatTypeTemplate(float f) : base(f){}
		/// <summary>
		/// stores a double as an XMLField in the FlatTypeTemplate
		/// </summary>
		/// <param name="d">the value of the field</param>
		public FlatTypeTemplate(double d):base(d){}
		/// <summary>
		/// stores a char as an XMLField in the FlatTypeTemplate
		/// </summary>
		/// <param name="c">the value of the field</param>
		public FlatTypeTemplate(char c):base(c){}
		/// <summary>
		/// stores a string as an XMLField in the FlatTypeTemplate
		/// </summary>
		/// <param name="s">the value of the field</param>
		public FlatTypeTemplate(String s):base(s){}
		/// <summary>
		/// stores a XMLTemplate  in the FlatTypeTemplate
		/// </summary>
		/// <param name="t">the XMLTemplate</param>
		public FlatTypeTemplate(XMLTemplate t):base(t){	}
		/// <summary>
		/// stores an IMatchable object as an XMLField in the FlatTypeTemplate
		/// </summary>
		/// <param name="m">the value of the field</param>
		public FlatTypeTemplate(IMatchable m):base(m){}
		/// <summary>
		/// stores an IXMLMatchable object as an XMLField in the FlatTypeTemplate
		/// </summary>
		/// <param name="m">the value of the field</param>
		public FlatTypeTemplate(IXMLMatchable m):base(m){}
		/// <summary>
		/// stores an object as an XMLField in the FlatTypeTemplate. In further execution the type is differentiated.
		/// </summary>
		/// <param name="o">the value of the field</param>
		public FlatTypeTemplate(Object o):base(o){	}
		/// <summary>
		/// stores an ITuple in the FlatTypeTemplate
		/// </summary>
		/// <param name="t">the ITuple</param>
		public FlatTypeTemplate(ITuple t):base(t) {}
		/// <summary>
		/// Stores an IField in the FlatTypeTemplate
		/// </summary>
		/// <param name="f">the IField</param>
		public FlatTypeTemplate(IField f):base(f){}
		/// <summary>
		/// the match 
		/// </summary>
		/// <param name="xmldoc">the XmlNode to match</param>
		/// <returns>true, if matching, false otherwise</returns>
		public override bool matches(XmlNode xmldoc) 
		{
			return match(this.getXmlDocument().DocumentElement, xmldoc);
		}
		/// <summary>
		/// the match 
		/// </summary>
		/// <param name="templatenode">the templatenode to match</param>
		/// <param name="tuplenode">the tuplenode to match</param>
		/// <returns>true, if matching, false otherwise</returns>
		public override bool match(XmlNode templatenode, XmlNode tuplenode) 
		{
			int tuplecounttemplate = gettuplecount(templatenode);
			int fieldcounttemplate = getfieldcount(templatenode);
			int tuplecounttuple = gettuplecount(tuplenode);
			int fieldcounttuple = getfieldcount(tuplenode);
			if((tuplecounttemplate==tuplecounttuple) &&(fieldcounttemplate==fieldcounttuple)) return true;
			return false;
		}
	}
}