using System;
using System.Xml;
using System.Xml.Schema;



namespace XMLTupleSpaceShared
{
	/// <summary>
	/// validates a xmlfile against a given xmlschema
	/// </summary>
	class Validator
	{
		/// <summary>
		/// validates a xmlfile against a given xmlschema
		/// </summary>
		/// <param name="schemauri">
		/// the xmlschema
		/// </param>
		/// <param name="xmlfile">
		/// the xmlfile
		/// </param>
		public void validate(string schemauri, string xmlfile) 
		{
			XmlValidatingReader reader = null;

			try 
			{
				XmlTextReader nvr = new XmlTextReader (xmlfile);
				nvr.WhitespaceHandling = WhitespaceHandling.None;

				reader = new XmlValidatingReader (nvr);
				reader.Schemas.Add (GetTargetNamespace (schemauri), schemauri);
				reader.ValidationEventHandler +=
					new ValidationEventHandler (OnValidationError);

				while (reader.Read ());
			}
			catch (Exception ex) 
			{
				Console.WriteLine (ex.Message);
			}
			finally 
			{
				if (reader != null)
					reader.Close ();
			}
		}

		/// <summary>
		/// handles errors in the validating process
		/// </summary>
		/// <param name="sender">the source of the error</param>
		/// <param name="e">the exception that has been thrown</param>
		static void OnValidationError (object sender, ValidationEventArgs e)
		{
			Console.WriteLine (e.Message);
		}
		/// <summary>
		/// gets the target namespace
		/// </summary>
		/// <param name="src">
		/// the sourcefile
		/// </param>
		/// <returns>
		/// returns the target namespace
		/// </returns>
		public static string GetTargetNamespace (string src)
		{
			XmlTextReader reader = null;

			try 
			{
				reader = new XmlTextReader (src);
				reader.WhitespaceHandling = WhitespaceHandling.None;

				while (reader.Read ()) 
				{
					if (reader.NodeType == XmlNodeType.Element &&
						reader.LocalName == "schema") 
					{
						while (reader.MoveToNextAttribute ()) 
						{
							if (reader.Name == "targetNamespace")
								return reader.Value;
						}
					}
				}
				return "";
			}
			finally 
			{
				if (reader != null)
					reader.Close ();
			}
		}
	}
}