using System;
using System.Xml;
using Interfaces;
using Helpers;

namespace XMLTupleSpaceShared
{
	/// <summary>
	/// Summary description for XMLField.
	/// </summary>
	[Serializable]
	public class XMLField : MarshalByRefObject,IField
	{
		/// <summary>
		/// contains the string reprsentation of the type of the Field
		/// </summary>
		private String type;
		/// <summary>
		/// contains the value of the Field
		/// </summary>
		private object fieldvalue;
		
		/// <summary>
		/// constructs a XMLField without an value
		/// </summary>
		public XMLField()
		{
			type = null;
			fieldvalue = null;
		}
		/// <summary>
		/// constructs an XMLField containing a byte
		/// </summary>
		/// <param name="b">the value of the XMLField</param>
		public XMLField(byte b)
		{
			type = b.GetType().ToString();
			fieldvalue = b;

		}
		/// <summary>
		/// constructs an XMLField containing a SByte
		/// </summary>
		/// <param name="sb">the value of the XMLField</param>
		public XMLField(SByte sb)
		{
			type = sb.GetType().ToString();
			fieldvalue = sb;
		}
		/// <summary>
		/// constructs an XMLField containing a short
		/// </summary>
		/// <param name="s">the value of the XMLField</param>
		public XMLField (short s)
		{
			type = s.GetType().ToString();
			fieldvalue = s;
		}
		/// <summary>
		/// constructs an XMLField containing an int
		/// </summary>
		/// <param name="i">the value of the XMLField</param>
		public XMLField(int i)
		{
			type = i.GetType().ToString();
			fieldvalue = i;
		}
		/// <summary>
		/// constructs an XMLField containing a long
		/// </summary>
		/// <param name="l">the value of the XMLField</param>
		public XMLField(long l)
		{
			type = l.GetType().ToString();
			fieldvalue = l;
		}
		/// <summary>
		/// constructs an XMLField containing a float
		/// </summary>
		/// <param name="f">the value of the XMLField</param>
		public XMLField(float f)
		{
			type = f.GetType().ToString();
			fieldvalue = f;
		}
		/// <summary>
		/// constructs an XMLField containing a double
		/// </summary>
		/// <param name="d">the value of the XMLField</param>
		public XMLField(Double d)
		{
			type = d.GetType().ToString();
			fieldvalue = d;
		}
		/// <summary>
		/// constructs an XMLField containing a char
		/// </summary>
		/// <param name="c">the value of the XMLField</param>
		public XMLField(Char c)
		{
			type = c.GetType().ToString();
			fieldvalue = c;
		}
		/// <summary>
		/// constructs an XMLField containing a string
		/// </summary>
		/// <param name="s">the value of the XMLField</param>
		public XMLField (String s)
		{
			type = s.GetType().ToString();
			fieldvalue = s;
		}
		/// <summary>
		/// constructs an XMLField containing an object 
		/// </summary>
		/// <param name="o">the value of the XMLField</param>
		// fr soap - objekte 
		public XMLField (Object o)
		{
			type = "Soap";
			ExtendedXmlDocument xmldoc = new ExtendedXmlDocument();
			XmlNode node = XMLTupleHelper.buildSoapNode(xmldoc, o);
			fieldvalue = node;
		}
		/// <summary>
		/// constructs an XMLField containing an object array
		/// </summary>
		/// <param name="o">the value of the XMLField</param>
		public XMLField(Object[] o)
		{
			type = "Soap";
			ExtendedXmlDocument xmldoc = new ExtendedXmlDocument();
			XmlNode node = XMLTupleHelper.buildSoapNode(xmldoc, o);
			fieldvalue = node;
		}
		/// <summary>
		/// constructs an XMLField containing an ExtendedXmlDocument
		/// </summary>
		/// <param name="doc">the value of the XMLField</param>
		public XMLField(ExtendedXmlDocument doc)
		{
			ExtendedXmlDocument xmldoc = new ExtendedXmlDocument();
			fieldvalue = XMLTupleHelper.buildXmlDocumentNode(xmldoc, doc);
		}
		/// <summary>
		/// presents the Type of the Field
		/// </summary>
		/// <param name="obj">
		/// the Type of the Field
		/// </param>
		/// <returns>
		/// returns the Type of the Field
		/// </returns>
		public String Type
		{
			get{return type;}
			set{this.type = value;}
		}
		/// <summary>
		/// contains the Value of the Field
		/// </summary>
		/// <param name="obj">
		/// the Value of the Field
		/// </param>
		/// <returns>
		/// returns the Value of the Field
		/// </returns>
		public object FieldValue
		{
			get{return fieldvalue;}
			set{this.fieldvalue = value;}
		}
		public object getObject()
		{
			if(fieldvalue is XmlNode)  
			{
				string soapString = ((XmlNode) fieldvalue).InnerXml;
				return Datatypes.castToObject(this.Type, soapString);
			}
			return Datatypes.castToObject(this.Type, this.FieldValue.ToString());
		}
		/// <summary>
		///  string representation of an Field
		/// </summary>
		/// <returns>
		/// returns the string representation of an Field
		/// </returns>
		public override string ToString()
		{
			return fieldvalue.ToString();
		}
	
	}
}
