using System;
using System.Xml;

namespace XMLTupleSpaceShared
{
	/// <summary>
	/// Class displays a pretty-looking version of the <c>XmlDocument</c>
	/// </summary>
	public class XMLPrinter
	{
		public XMLPrinter()
		{
			
		}

		/// <summary>
		/// formats the xml-document
		/// </summary>
		/// <param name="nodelist">the XmlDocument's nodes</param>
		/// <returns>the string-representation of the XmlDocument</returns>
		public string printXML(XmlNodeList nodelist) 
		{
			return GetChildNodes(nodelist, 0);
		}
		/// <summary>
		/// formats the xml-document
		/// </summary>
		/// <param name="nodelist">the xmldocument's nodes</param>
		/// <param name="i">indentation level</param>
		/// <returns>the string</returns>
		public string printXML(XmlNodeList nodelist, int i) 
		{
			return GetChildNodes(nodelist, i);
		}


		/// <summary>
		/// builds the string representing childnodes of an XmlNode
		/// </summary>
		/// <param name="objNodeList">the XmlNodes</param>
		/// <param name="intLevel">current indentation level</param>
		/// <returns>string representation of childnodes</returns>
		string GetChildNodes(XmlNodeList objNodeList, int intLevel)
		{
			string strNodes = "";
			foreach (XmlNode objNode in objNodeList)
			{
				strNodes = strNodes + "\n" + GetIndent(intLevel)
					+ GetNodeType(objNode.NodeType) + " : " + objNode.Name;
				if (objNode.NodeType == XmlNodeType.XmlDeclaration)
				{
					XmlDeclaration objXmlDec = (XmlDeclaration)objNode;
					strNodes = strNodes + "version = " + objXmlDec.Version
						+ " standalone = " + objXmlDec.Standalone;
				}
				else
				{
					strNodes = strNodes + " value = " + objNode.Value + "\n";
				}

				if (objNode.NodeType == XmlNodeType.Element)
				{
					foreach (XmlAttribute objAttr in objNode.Attributes)
					{
						strNodes = strNodes + GetIndent(intLevel + 1)
							+ GetNodeType(objAttr.NodeType) + " : " + objAttr.Name
							+ " value = " + objAttr.Value + "\n";
					}
				}
				if (objNode.HasChildNodes)
					strNodes = strNodes + GetChildNodes(objNode.ChildNodes, intLevel + 1) ;
		
			}// go to next node
			return strNodes;   // pass the result back to the caller
		}

		/// <summary>
		/// keeps track of indentation level
		/// </summary>
		/// <param name="intLevel">the current indentation level</param>
		/// <returns>string with indentation</returns>
		string GetIndent(int intLevel)
		{
			// returns a string of non-breaking spaces used to indent each line
			string strIndent = "";
			for (int intIndent = 0; intIndent <= intLevel; intIndent++)
				//strIndent = strIndent + "&nbsp; &nbsp; &nbsp; ";
				strIndent = strIndent + "  ";
			return strIndent;
		}


		/// <summary>
		/// A node type switcher
		/// </summary>
		/// <param name="intType">integer constant representing a node's type</param>
		/// <returns>the string representation of the type</returns>
		string GetNodeType(XmlNodeType intType)
		{
			// returns the node type as a string
			switch (Convert.ToInt32(intType))
			{
				case 0: return "NONE";
				case 1: return "ELEMENT"; 
				case 2: return "ATTRIBUTE"; 
				case 3: return "TEXT"; 
				case 4: return "CDATA SECTION"; 
				case 5: return "ENTITY REFERENCE"; 
				case 6: return "ENTITY"; 
				case 7: return "PROCESSING INSTRUCTION"; 
				case 8: return "COMMENT"; 
				case 9: return "DOCUMENT"; 
				case 10: return "DOCUMENT TYPE"; 
				case 11: return "DOCUMENT FRAGMENT"; 
				case 12: return "NOTATION"; 
				case 13: return "WHITESPACE"; 
				case 14: return "SIGNIFICANT WHITESPACE"; 
				case 15: return "END ELEMENT"; 
				case 16: return "END ENTITY"; 
				case 17: return "Xml DECLARATION"; 
				case 18: return "NODE (ALL)"; 
				default: return "UNKNOWN"; 
			}
		
		}
	}
}