using System;
using System.Xml;
using System.Collections;
using System.Threading;
using System.Runtime.CompilerServices;
using Interfaces;
using System.IO;
using System.Net;
using System.Runtime.Remoting.Lifetime;

namespace XMLTupleSpaceShared
{
	/// <summary>
	/// the main class for the tuplespace
	/// </summary>
	public class XMLServer : MarshalByRefObject
	{
		//private static readonly ILog logger = LogManager.GetLogger(typeof(XMLServer));
		//private FileInfo fnfo = new FileInfo("..\\logconfig.xml");
		//private static int servercreated = 0;
		/// <summary>
		/// the set of tuplespaces on this server
		/// </summary>
		private ArrayList tuplespaces = new ArrayList();

		/// <summary>
		/// a new Server is created
		/// </summary>
		public XMLServer()
		{
		}
		
		/// <summary>
		/// String representation of the TupleBucket(s)
		/// </summary>
		/// <param name="tbucket">
		/// the name of the TupleBucket
		/// </param>
		public void toString(String tbucket) 
		{
			IEnumerator enumerator = tuplespaces.GetEnumerator();
			while(enumerator.MoveNext()) 
			{
				XMLTupleBucket tb = (XMLTupleBucket) enumerator.Current;
				if(tb.Name.Equals(tbucket)) 
				{
					Console.WriteLine("Server contains : " + tb.ToString() );
				}
			}

		}
		/// <summary>
		/// tries to create a tuplespace. if name already exists the creation fails and method will return null
		/// </summary>
		/// <param name="tsName"></param>
		/// <returns>Either a new TupleSpace or an existing one with </returns>
		[MethodImpl(MethodImplOptions.Synchronized)] 
		public XMLTupleSpace createTSpace(String tsName) 
		{		
			if(!exists(tsName)) 
			{
				tuplespaces.Add(new XMLTupleBucket(tsName));
				XMLTupleSpace ts = new XMLTupleSpace(tsName, this);
				return ts;
			} 
			else 
			{	
				return null;
			}
		}

		/// <summary>
		/// tries to connect to tuplespace. if no tuplespace with that name exists, method will return null
		/// </summary>
		/// <param name="tsName"></param>
		/// <returns></returns>
		[MethodImpl(MethodImplOptions.Synchronized)] 
		public XMLTupleSpace connectTSpace(String tsName) 
		{
			if(exists(tsName)) 
			{
				XMLTupleSpace ts = new XMLTupleSpace(tsName, this);
				return ts;
			} 
			else 
			{				
				return null;
			}
		}

		/// <summary>
		/// checks if a tuplespace with name tsName already exists
		/// </summary>
		/// <param name="tsName"></param>
		/// <returns></returns>
		/// 
		[MethodImpl(MethodImplOptions.Synchronized)] 
		private bool exists(String tsName) 
		{
			IEnumerator enumerator = tuplespaces.GetEnumerator();
			while(enumerator.MoveNext()) 
			{
				XMLTupleBucket tb = (XMLTupleBucket) enumerator.Current;
				if(tb.Name.Equals(tsName)) 
				{
					return true;
				}
			}
			return false;
		}
		/// <summary>
		/// tries to destroy tuplespace with given name
		/// </summary>
		/// <param name="tsName"></param>
		// destroy tuplespace with name tsName
		// ought to be better if method returns boolean value
		[MethodImpl(MethodImplOptions.Synchronized)] 
		public void destroyTupleSpace(String tsName) 
		{			
			IEnumerator enumerator = tuplespaces.GetEnumerator();
			while(enumerator.MoveNext()) 
			{
				XMLTupleBucket tb = (XMLTupleBucket) enumerator.Current;
				if(tb.Name.Equals(tsName)) 
				{
					tuplespaces.Remove(tb);
					//logger.Debug("Destroyed tuplespace : " + tsName);
				}
			}
		}

		/// <summary>
		/// tries to add a tuple to named tuplespace
		/// </summary>
		/// <param name="tsName"></param>
		/// <param name="t"></param>
		[MethodImpl(MethodImplOptions.Synchronized)] 
		public void addToTs(String tsName, ITuple t)
		{
			IEnumerator e = tuplespaces.GetEnumerator();
			while (e.MoveNext())
			{	
				XMLTupleBucket tb = (XMLTupleBucket) e.Current;
				if(tb.Name.Equals(tsName))
				{
					tb.add(t);
				}
			}
		}
		/// <summary>
		/// tries to add a tuple to named tuplespace
		/// </summary>
		/// <param name="tsName">the tuplespace to work in</param>
		/// <param name="t">the tuple to add</param>
		[MethodImpl(MethodImplOptions.Synchronized)] 
		public void addToTs(String tsName, XMLTemplate t)
		{
			IEnumerator e = tuplespaces.GetEnumerator();
			while (e.MoveNext())
			{	
				XMLTupleBucket tb = (XMLTupleBucket) e.Current;
				if(tb.Name.Equals(tsName))
				{
					tb.add(t);
				}
			}
		}

		/// <summary>
		/// tries to read and remove a tuple from tuplespace
		/// </summary>
		/// <param name="tsName">the tuplespace to search in</param>
		/// <param name="t">the tuple to remove</param>
		/// <returns>the tuple if a match was found</returns>
		[MethodImpl(MethodImplOptions.Synchronized)] 
		public ITuple syncRemove (String tsName, ITuple t)
		{			
			ITuple returnedTuple = syncRemove(tsName, new DeepTemplate(t));
			return returnedTuple;
		}
		/// <summary>
		/// reads a tuple from the tuplespace without removing it
		/// </summary>
		/// <param name="tsName">the tuplespace to search in</param>
		/// <param name="t">the tuple to search</param>
		/// <returns>the tuple if it was found</returns>
		[MethodImpl(MethodImplOptions.Synchronized)] 
		public ITuple syncReadFromTS (String tsName, ITuple t)
		{			
			ITuple returnedTuple = syncRemove(tsName, new DeepTemplate(t));
			return returnedTuple;
		}

		/// <summary>
		/// read a tuple that matches to the template from the tuplespace
		/// </summary>
		/// <param name="tsName">the tuplespace to search in</param>
		/// <param name="t">the template</param>
		/// <returns>a tuple that matches the template</returns>
		[MethodImpl(MethodImplOptions.Synchronized)] 
		public ITuple syncReadFromTS(String tsName, XMLTemplate t) 
		{
			IEnumerator e = tuplespaces.GetEnumerator();			
			while (e.MoveNext())
			{	
				XMLTupleBucket tb = (XMLTupleBucket) e.Current;
				if(tb.Name.Equals(tsName))
				{
					XmlNodeList xmlnlist = XMLTupleHandler.findTuples(tb.getTupleBucket(), t);
					//logger.Debug("number of potential matches : " + xmlnlist.Count);					
					foreach(XmlNode n in xmlnlist)
					{
						if(t.matches(n)) 
						{
							ITuple tuple = new XMLTuple();
							((XMLTuple) tuple).getXmlDocument().LoadXml(n.OuterXml);							
							return tuple;
						}
					}
					return null;
				}
			}
			return null;
		}

		/// <summary>
		/// reads and removes a tuple matching the template
		/// </summary>
		/// <param name="tsName">the tuplespace to search in</param>
		/// <param name="t">the template</param>
		/// <returns>a tuple matching the template</returns>
		public ITuple syncRemove(String tsName, XMLTemplate t) 
		{
			IEnumerator e = tuplespaces.GetEnumerator();			
			while (e.MoveNext())
			{	
				XMLTupleBucket tb = (XMLTupleBucket) e.Current;
				if(tb.Name.Equals(tsName))
				{
					XmlNodeList xmlnlist = XMLTupleHandler.findTuples(tb.getTupleBucket(), t);
					foreach(XmlNode n in xmlnlist)
					{
						if(t.matches(n)) 
						{
							ITuple tuple = tb.remove(n);
							return tuple;
						}
					}
					return null;
				}
			}
			return null;
		}

	}
}
