using System;
using System.Xml;
using Interfaces;
using System.Collections;
using System.Runtime.Serialization.Formatters.Soap;
//using log4net;
//using log4net.Config;
using System.IO;
using Helpers;


namespace XMLTupleSpaceShared
{
	/// <summary>
	/// the Template for any Field
	/// </summary>
	[Serializable]
	public struct FieldTemplate{};
	/// <summary>
	/// the Template for any Tuple
	/// </summary>
	[Serializable]
	public struct TupleTemplate{};
	/// <summary>
	/// represents a template for a tuple
	/// </summary>
	public abstract class XMLTemplate: XMLTuple
	{
//		protected static readonly ILog logger = LogManager.GetLogger(typeof(XMLTemplate));
			protected static readonly string UNEQUAL = "unequal";
		
		/// <summary>
		/// creates an empty XMLTemplate
		/// </summary>
		public  XMLTemplate()
		{
			xmldoc = XMLTupleHelper.createRootDocumentForTemplate(this);
		}
		/// <summary>
		/// stores a FieldTemplate in the XMLTemplate
		/// </summary>
		/// <param name="fieldTemplate">the fieldTemplate</param>
		public XMLTemplate(FieldTemplate fieldTemplate)
		{
			xmldoc = XMLTupleHelper.createRootDocumentForTemplate(this);
			add(fieldTemplate);			
		 }
		/// <summary>
		/// stores a TupleTemplate in the XMLTemplate
		/// </summary>
		/// <param name="tupleTemplate">the tupleTemplate</param>
		public XMLTemplate(TupleTemplate tupleTemplate)
		{
			xmldoc = XMLTupleHelper.createRootDocumentForTemplate(this);
			add(tupleTemplate);			
		}
		/// <summary>
		/// stores a IMatchable as an XMLField in the XMLTemplate
		/// </summary>
		/// <param name="m">the value of the field</param>
		public XMLTemplate(IMatchable m)
		{
			xmldoc = XMLTupleHelper.createRootDocumentForTemplate(this);
			add(m);			
		}
		/// <summary>
		/// stores a IXMLMatchable as an XMLField in the XMLTemplate
		/// </summary>
		/// <param name="m">the value of the field</param>
		public XMLTemplate(IXMLMatchable m)
		{
			xmldoc = XMLTupleHelper.createRootDocumentForTemplate(this);
			add(m);
		}
		/// <summary>
		/// stores an int as an XMLField in the XMLTemplate
		/// </summary>
		/// <param name="i">the value of the field</param>
		public XMLTemplate(int i)
		{
			xmldoc = XMLTupleHelper.createRootDocumentForTemplate(this);
			field = new XMLField(i);
			add(field);
		}
		/// <summary>
		/// stores a byte as an XMLField in the XMLTemplate
		/// </summary>
		/// <param name="b">the value of the field</param>
		public XMLTemplate(byte b)
		{
			xmldoc = XMLTupleHelper.createRootDocumentForTemplate(this);
			field = new XMLField(b);
			add(field);
		}
		/// <summary>
		/// stores a SByte as an XMLField in the XMLTemplate
		/// </summary>
		/// <param name="b">the value of the field</param>
		public XMLTemplate(SByte sb) 
		{
			xmldoc = XMLTupleHelper.createRootDocumentForTemplate(this);
			field = new XMLField(sb);
			add(field);
		}
		/// <summary>
		/// stores a short as an XMLField in the XMLTemplate
		/// </summary>
		/// <param name="s">the value of the field</param>
		public XMLTemplate(short s) 
		{
			xmldoc = XMLTupleHelper.createRootDocumentForTemplate(this);
			field = new XMLField(s);
			add(field);
		}
		/// <summary>
		/// stores a long as an XMLField in the XMLTemplate
		/// </summary>
		/// <param name="l">the value of the field</param>
		public XMLTemplate(long l)
		{
			xmldoc = XMLTupleHelper.createRootDocumentForTemplate(this);
			field = new XMLField(l);
			add(field);
		}
		/// <summary>
		/// stores a float as an XMLField in the XMLTemplate
		/// </summary>
		/// <param name="f">the value of the field</param>
		public XMLTemplate(float f) 
		{
			xmldoc = XMLTupleHelper.createRootDocumentForTemplate(this);
			field = new XMLField(f);
			add(field);
		}
		/// <summary>
		/// stores a doublee as an XMLField in the XMLTemplate
		/// </summary>
		/// <param name="d">the value of the field</param>
		public XMLTemplate(double d)
		{
			xmldoc = XMLTupleHelper.createRootDocumentForTemplate(this);
			field = new XMLField(d);
			add(field);
		}
		/// <summary>
		/// stores a char as an XMLField in the XMLTemplate
		/// </summary>
		/// <param name="c">the value of the field</param>
		public XMLTemplate(char c)
		{
			xmldoc = XMLTupleHelper.createRootDocumentForTemplate(this);
			field = new XMLField(c);
			add(field);
		}
		/// <summary>
		/// stores a string as an XMLField in the XMLTemplate
		/// </summary>
		/// <param name="s">the value of the field</param>
		public XMLTemplate(String s)
		{
			xmldoc = XMLTupleHelper.createRootDocumentForTemplate(this);
			field = new XMLField(s);
			add(field);
		}
		/// <summary>
		/// Stores an object as a XMLField in the XMLTemplate. In further execution the type is differentiated.
		/// </summary>
		/// <param name="o">the value of the field</param>
		public XMLTemplate(Object o)
		{
			xmldoc = XMLTupleHelper.createRootDocumentForTemplate(this);
			field = new XMLField(o);
			add(field);
		}

		/// <summary>
		/// stores a ITuple in the template
		/// </summary>
		/// <param name="t">the ITuple</param>
		public XMLTemplate(ITuple t)
		{
			xmldoc.LoadXml(((XMLTuple) t).getXmlDocument().DocumentElement.OuterXml);
		}
		/// <summary>
		/// Stores a field in the template
		/// </summary>
		/// <param name="f">the field</param>
		public XMLTemplate(IField f)
		{
			xmldoc = XMLTupleHelper.createRootDocumentForTemplate(this);
			add(f);
		}	

		/// <summary>
		/// adds a template to the current one
		/// </summary>
		/// <param name="t">the template</param>
		public void add(XMLTemplate t)							//<- noch drum kmmern
		{
			XmlNode newNode = XMLTupleHelper.convertToXML(xmldoc, t);
			xmldoc.DocumentElement.AppendChild(newNode);
			increasecount();
		}
		/// <summary>
		/// adds a FieldTemplate to the current XMLTemplate
		/// </summary>
		/// <param name="fieldTemplate">the FieldTemplate</param>
		public void add(FieldTemplate fieldTemplate)
		{
			XmlNode newElem = xmldoc.CreateElement("FieldTemplate");
			xmldoc.DocumentElement.AppendChild(newElem);
			incFieldcount();
		}
		/// <summary>
		/// adds a TupleTemplate to the current XMLTemplate
		/// </summary>
		/// <param name="tupleTemplate">the TupleTemplate</param>
		public void add(TupleTemplate tupleTemplate)
		{
			XmlNode newElem = xmldoc.CreateElement("TupleTemplate");
			xmldoc.DocumentElement.AppendChild(newElem);
			increasecount();
		}
		
		/// <summary>
		/// adds an object with IMatchable interface to the template
		/// </summary>
		/// <param name="m">the object which implement the IMatchable interface</param>
		public void add(IMatchable m)
		{
			XmlNode newNode = XMLTupleHelper.convertFieldToXML(xmldoc, m);
			xmldoc.DocumentElement.AppendChild(newNode);
			incFieldcount();
		}
		/// <summary>
		/// adds an object with IXMLMatchable interface to the template
		/// </summary>
		/// <param name="m">the object which implement the IXMLMatchable interface</param>
		public void add(IXMLMatchable m)
		{
			XmlNode newNode = XMLTupleHelper.convertFieldToXML(xmldoc, m);
			xmldoc.DocumentElement.AppendChild(newNode);
			incFieldcount();
		}
		/// <summary>
		/// renames the type-attribut of the XMLTuple
		/// </summary>
		/// <param name="t">the ITuple to rename</param>
		/// <param name="newType">the new type</param>
		public void renameType(ITuple t, String newType)
		{
			XmlNode n = ((XMLTuple)t).getXmlDocument().DocumentElement;
			((XmlElement)n).SetAttribute("type", newType);
		}

		/// <summary>
		/// the spezialisations of the XMLTemplate have to implement the match
		/// </summary>
		/// <param name="templatenode">the Templatenode</param>
		/// <param name="tuplenode">the Tuplenode</param>
		/// <returns>true, if matching, false otherwise</returns>
		public abstract bool matches(XmlNode xmldoc);
		/// <summary>
		/// the spezialisations of the XMLTemplate have to implement the match
		/// </summary>
		/// <param name="templatenode">the Templatenode</param>
		/// <param name="tuplenode">the Tuplenode</param>
		/// <returns>true, if matching, false otherwise</returns>
		public abstract bool match(XmlNode templatenode, XmlNode tuplenode);
		/// <summary>
		/// deserializes the IXMLMatchable object and performs the operation by calling the object's matches() method
		/// </summary>
		/// <param name="templateNode">the node containing the serialized IMatchable object</param>
		/// <param name="tupleNode">the node containing the object to match against</param>
		/// <returns>true if object's matches method returns true, false else</returns>
		protected bool matchOnIXMLMatchable(XmlNode templateNode, XmlNode tupleNode) 
		{
			IXMLMatchable matcher = (IXMLMatchable) Datatypes.castToObject(templateNode);		
			XmlDocument doc = new XmlDocument();
			doc.LoadXml(tupleNode.InnerXml);
			return matcher.matches(doc);
		}

		/// <summary>
		/// deserializes the IMatchable object and performs the operation by calling the object's matches() method
		/// </summary>
		/// <param name="templateNode">the node containing the serialized IMatchable object</param>
		/// <param name="tupleNode">the node containing the object to match against</param>
		/// <returns>true if object's matches method returns true, false else</returns>
		protected bool matchOnIMatchable(XmlNode templateNode, XmlNode tupleNode) 
		{
			IMatchable matcher = (IMatchable) Datatypes.castToObject(templateNode);
			Object n = Datatypes.castToObject(tupleNode);
			return matcher.matches(n);			
		}
		/// <summary>
		/// compares the fieldcount and tuplecount attributes of a tuple-element
		/// </summary>
		/// <param name="fst">first xmlnode</param>
		/// <param name="snd">second xmlnode</param>
		/// <returns>true if the attribute-values match, false else</returns>
		protected bool compareCount(XmlNode fst, XmlNode snd)
		{
			int fst_fieldcount = Int32.Parse(((XmlElement)fst).GetAttribute("fieldcount"));
			int fst_tuplecount = Int32.Parse(((XmlElement)fst).GetAttribute("tuplecount"));
			int snd_fieldcount = Int32.Parse(((XmlElement)snd).GetAttribute("fieldcount"));
			int snd_tuplecount = Int32.Parse(((XmlElement)snd).GetAttribute("tuplecount"));
			if(fst_fieldcount==snd_fieldcount && fst_tuplecount==snd_tuplecount) return true;
			else return false;
		}

		/// <summary>
		/// compares the name of xmlnodes (an xml-tag's name)
		/// </summary>
		/// <param name="fst">first xmlnode</param>
		/// <param name="snd">second xmlnode</param>
		/// <returns>the name if both are equal, UNEQUAL else</returns>
		protected string elementNameMatch(XmlNode fst, XmlNode snd) 
		{
			if(fst.Name.Equals(snd.Name))
				return fst.Name;
			else return UNEQUAL;
		}

		/// <summary>
		/// compares the contents, i.e. the text inside of an xml-element
		/// </summary>
		/// <param name="fst">the first XmlNode</param>
		/// <param name="snd">the second XmlNode</param>
		/// <returns>value of the XmlNodes if valuess match, "unequal" if they don't.</returns>
		protected string valueMatch(XmlNode fst, XmlNode snd) 
		{
			try 
			{		
				if(fst.Value.Equals(snd.Value))
					return fst.Value;
				else return UNEQUAL;
			} 
			catch(NullReferenceException nref) 
			{
				if(fst.Value == null && snd.Value == null) 
				{
					return "";
				}
				else 
				{
					return UNEQUAL;
				}
			}
		}

		/// <summary>
		/// checks if "type" - attribute of two nodes match
		/// </summary>
		/// <param name="fst">first xmlnode</param>
		/// <param name="snd">second xmlnode</param>
		/// <returns>value of the "type" - attribute if types match, "unequal" if they don't.</returns>
		protected string typematch(XmlNode fst, XmlNode snd)
		{	
			String fst_type = ((XmlElement)fst).GetAttribute("type");
			String snd_type = ((XmlElement)snd).GetAttribute("type");
			if(fst_type.Equals(snd_type)) 
			{
				if(fst_type.Equals("soap")) 
				{
					return "soap";					
				}
				return fst_type;
			}
			else if(fst_type.Equals("IMatchable")) 
			{
				return "IMatchable";
			}
			else if(fst_type.Equals("IXMLMatchable"))
				return "IXMLMatchable";
			else return UNEQUAL;
		}
		/// <summary>
		/// performs the match of a tuple and a TupleTemplate
		/// </summary>
		/// <param name="fst">the tuple as a XmlNode</param>
		/// <param name="snd">the TupleTemplate as XmlNode</param>
		/// <returns>true, if matching, false otherwise</returns>
		protected bool tupleTemplatematch(XmlNode fst, XmlNode snd)
		{
			if (fst.Name.Equals("Tuple")) return true;
			else return false;
		}
		/// <summary>
		/// performs the match of a Field and a FieldTemplate
		/// </summary>
		/// <param name="fst">the field as a XmlNode</param>
		/// <param name="snd">the fieldTemplate as XmlNode</param>
		/// <returns>true, if matching, false otherwise</returns>
		protected bool fieldTemplatematch(XmlNode fst, XmlNode snd)
		{
			if (fst.Name.Equals("Field")) return true;
			else return false;
		}
		/// <summary>
		/// gets the numer of tuples of the current template
		/// </summary>
		/// <param name="node">the template as XmlNode</param>
		/// <returns>the number of tuples of the current template</returns>
		protected int gettuplecount(XmlNode node)
		{
			int count =Int32.Parse( ((XmlElement)node).GetAttribute("tuplecount"));
			return count;
		}
		/// <summary>
		/// gets the numer of fields of the current template
		/// </summary>
		/// <param name="node">the template as XmlNode</param>
		/// <returns>the number of fields of the current template</returns>
		protected int getfieldcount(XmlNode node)
		{
			int count =Int32.Parse( ((XmlElement)node).GetAttribute("fieldcount"));
			return count;
		}
		/// <summary>
		/// performs match of fields. 
		/// </summary>
		/// <param name="fstfield">the first Field to match</param>
		/// <param name="sndfield">the second Field to match</param>
		/// <returns>true, if matching, false otherwise</returns>
		protected bool matchFields (XmlNode fstfield, XmlNode sndfield)
		{
			string  fstType = ((XmlElement)fstfield).GetAttribute("type");
			string  sndType = ((XmlElement)sndfield).GetAttribute("type");
			#region IMatchable and IXMLMatchable
			string fieldType = typematch(fstfield, sndfield);
			if(fieldType.Equals(UNEQUAL)) return false;
			else if (fieldType.Equals("IMatchable")) 
			{
				return matchOnIMatchable(fstfield, sndfield);
			}
			else if(fieldType.Equals("IXMLMatchable")) 
			{
				return matchOnIXMLMatchable(fstfield, sndfield);
			}
				#endregion
				#region soap
			else if(fieldType.Equals("soap")) 
			{
				return soapMatch(fstfield.FirstChild, sndfield.FirstChild);
			} 
				#endregion
			else 
			{
				bool  values_match =((XmlElement)fstfield).InnerXml.Trim().Equals(((XmlElement)sndfield).InnerXml.Trim());
				if((fstType.Equals(sndType)) && (values_match)) return true; 
				else return false; // stimmen nicht berein
			}
		}


		/// <summary>
		/// performs a match two object's representation in soap. the format is given by the SoapFormatter.
		/// </summary>
		/// <param name="fst">the first node to match</param>
		/// <param name="snd">the second node to match</param>
		/// <returns>true, if matching, false otherwise</returns>
		protected bool soapMatch(XmlNode fst, XmlNode snd) 
		{			
			if(fst.ChildNodes.Count != snd.ChildNodes.Count) 
			{
				return false;
			}
			else 
			{
				IEnumerator fstEnumerator = fst.ChildNodes.GetEnumerator();
				IEnumerator sndEnumerator = snd.ChildNodes.GetEnumerator();
				while(fstEnumerator.MoveNext() && sndEnumerator.MoveNext()) 
				{
					XmlNode fst_current = (XmlNode) fstEnumerator.Current;
					XmlNode snd_current = (XmlNode) sndEnumerator.Current;
					string names_match = elementNameMatch(fst_current, snd_current);
					string values_match = valueMatch(fst_current, snd_current);
					if(!names_match.Equals(UNEQUAL) && !values_match.Equals(UNEQUAL))
					{
						//match children
						if(fst_current.ChildNodes.Count == 0) return true;
						else return soapMatch(fst_current, snd_current);
					}
					else 
					{
						return false;
					}
				}
			}
			return false;
		}
	}
}
