using System;
using System.Xml;
using Interfaces;
using System.IO;

namespace XMLTupleSpaceShared 
{
	/// <summary>
	/// stores all tuples (actual tuplespace)
	/// </summary>
	public class XMLTupleBucket : MarshalByRefObject, ITupleBucket
	{
		//private static readonly ILog logger = LogManager.GetLogger(typeof(XMLTupleBucket));
		//private FileInfo fnfo = new FileInfo("..\\..\\logconfig.xml");
		/// <summary>
		/// the bucket's name
		/// </summary>
		private string bucketname = "Defaultname";
		/// <summary>
		/// internal representation of the collection of tuples
		/// </summary>
		private ExtendedXmlDocument bucket;
		/// <summary>
		/// counter for tuples
		/// </summary>
		private int tuplecount = 1;

		/// <summary>
		///creates a tuplebucket
		/// </summary>
		/// <param name="str">the bucket's name</param>
		public XMLTupleBucket(String str)
		{
			//DOMConfigurator.Configure(fnfo);
			bucketname = str;
			bucket = XMLTupleHelper.createRootDocumentforBucket();			
		}

		/// <summary>
		/// gets / sets the name of the tuplebucket
		/// </summary>
		public string Name
		{ 
			get{ return bucketname;}
			set{ this.bucketname = value;}
		}

		/// <summary>
		/// adds a tuple to the bucket
		/// </summary>
		/// <param name="t">the tuple to add</param>
		public void add(ITuple t)
		{
			XmlNode newNode = XMLTupleHelper.convertToXML(bucket, t);
			bucket.DocumentElement.AppendChild(newNode);
			increasecount();
		}
		/// <summary>
		/// adds a XMLTemplate to the bucket
		/// </summary>
		/// <param name="t">the template to add</param>
		public void add(XMLTemplate t)
		{
			XmlNode newNode = XMLTupleHelper.convertToXML(bucket,t);
			bucket.DocumentElement.AppendChild(newNode);
			increasecount();
		}


		/// <summary>
		/// removes a tuple from the bucket
		/// </summary>
		/// <param name="t">the tuple to remove</param>
		/// <returns>the tuple if exists</returns>
		public ITuple remove(ITuple t)
		{
			XmlNode n = ((XMLTuple) t).getXmlDocument().FirstChild;
			return remove(n);
		}
		/// <summary>
		/// removes a XmlNode from the bucket
		/// </summary>
		/// <param name="n">the XmlNode to remove</param>
		/// <returns>the XMLTuple, if exists</returns>
		public ITuple remove(XmlNode n) 
		{
			XMLTuple t = new XMLTuple();
			t.getXmlDocument().LoadXml(n.OuterXml);
			try 
			{
				bucket.DocumentElement.RemoveChild(n);
			} 
			catch(ArgumentException) 
			{
				// n is not a childnode in bucket
				return null;
			}
			decreasecount();
			return t;
		}
		/// <summary>
		/// experimental for removal and identification by id
		/// </summary>
		/// <param name="i"></param>
		/// <returns></returns>
		public ITuple remove (int i)
		{
			return null;
		}
		
		/// <summary>
		/// decrease the counter for tuples (in class and xmldoc)
		/// </summary>
		private void decreasecount()
		{
			int counter = Int32.Parse(bucket.DocumentElement.GetAttribute("tuplecount")) -1;
			((XmlElement) bucket.DocumentElement).SetAttribute("tuplecount", counter.ToString());			
		}

		/// <summary>
		/// increase the counter for tuples (in class and xmldoc)
		/// </summary>
		private void increasecount() 
		{
			//increase count in xml and counter
			((XmlElement) bucket.DocumentElement).SetAttribute("tuplecount", tuplecount.ToString());
			tuplecount++;
		}

		/// <summary>
		/// provides a pretty-printed representation to the bucket
		/// </summary>
		/// <returns>the string reprsentation of the bucket</returns>
		public override String ToString()
		{
			XMLPrinter mpr = new XMLPrinter();
			return mpr.printXML(bucket.ChildNodes);
		}

		/// <summary>
		/// writes the contents of the bucket to disk
		/// </summary>
		/// <param name="name">the name of the file</param>
		public void writeToFile(String name) 
		{
			XmlTextWriter xmltwr = new XmlTextWriter(name, System.Text.Encoding.UTF8);
			bucket.Save(xmltwr);
			xmltwr.Flush();
			xmltwr.Close();
		}
		/// <summary>
		/// gets the tuplebucket 
		/// </summary>
		/// <returns>the tuplebucket</returns>
		public ExtendedXmlDocument getTupleBucket()
		{
			return bucket;
		}

	}
}
