using System;
using System.Xml;
using System.Collections;
using System.Runtime.Serialization.Formatters.Soap;
using System.Runtime.Serialization;
using System.Xml.XPath;
using System.IO;
using Interfaces;
using Helpers;
//using log4net;
//using log4net.Config;
using MatchingRelations;

namespace XMLTupleSpaceShared
{
	/// <summary>
	/// utility class for converting the objects in a xml representation
	/// </summary>

	public class XMLTupleHelper : MarshalByRefObject
	{
		/// <summary>
		/// string template for XMLTuple
		/// </summary>
		private static string xmltemplate = "<?xml version=\"1.0\" encoding=\"UTF-8\"?><Tuple><Tuple/></Tuple>";
		/// <summary>
		/// string template for XMLtemplate
		/// </summary>
		private static string xmlfortemplate = "<?xml version=\"1.0\" encoding=\"UTF-8\"?><Template><Tuple/></Template>";
		/// <summary>
		/// string template for XMLTupleBucket
		/// </summary>
		private static string buckettemplate = "<?xml version=\"1.0\" encoding=\"UTF-8\"?><TupleSpace><Tuple/></TupleSpace>";
		
//		private static readonly ILog logger = LogManager.GetLogger(typeof(XMLTupleHelper));
//		private static FileInfo fnfo = new FileInfo("..\\logconfig.xml");
//
//		public XMLTupleHelper()
//		{
////			DOMConfigurator.Configure(fnfo);
//		}

		///<summary>
		///Method adds a tuple to a xml document (empty or not...)
		///</summary>
		///<returns>the created Xmldoc</returns>
		public static ExtendedXmlDocument createRootDocument() 
		{
			ExtendedXmlDocument xmldoc = new ExtendedXmlDocument();
			xmldoc.LoadXml(xmltemplate);
			XmlNode root = xmldoc.DocumentElement;
			root.RemoveAll();

			((XmlElement) root).SetAttribute("type", "Tuple");
			((XmlElement) root).SetAttribute("tuplecount", "0");
			((XmlElement) root).SetAttribute("fieldcount", "0");
			
			return xmldoc;
		}
		/// <summary>
		/// Method adds a template to a xml document 
		/// </summary>
		/// <param name="t">the template to add</param>
		/// <returns>the created Xmldoc</returns>
		public static ExtendedXmlDocument createRootDocumentForTemplate(XMLTemplate t) 
		{
			ExtendedXmlDocument xmldoc = new ExtendedXmlDocument();
			xmldoc.LoadXml(xmlfortemplate);
			XmlNode root = xmldoc.DocumentElement;
			root.RemoveAll();

			((XmlElement) root).SetAttribute("type", t.GetType().ToString());
			((XmlElement) root).SetAttribute("tuplecount", "0");
			((XmlElement) root).SetAttribute("fieldcount", "0");
			
			return xmldoc;
		}
		/// <summary>
		///  creates an Xmldoc for the tuplebucket
		/// </summary>
		/// <returns> the created Xmldoc</returns>
		public static ExtendedXmlDocument createRootDocumentforBucket() 
		{
			ExtendedXmlDocument xmldoc = new ExtendedXmlDocument();
			xmldoc.LoadXml(buckettemplate);
			XmlNode root = xmldoc.DocumentElement;
			root.RemoveAll();

			((XmlElement) root).SetAttribute("root", "true");
			((XmlElement) root).SetAttribute("type", "XMLTupleBucket");
			((XmlElement) root).SetAttribute("tuplecount", "0");
			
			return xmldoc;
		}
		// Primitives.isPrimitive(obj) checks if obj is primitive datatype or not
		/// <summary>
		/// Converts an object, i.e. primitive datatypes and all more complex types to an xml-representation
		/// </summary>
		/// <param name="xmldoc">the XmlDocument to add this object to</param>
		/// <param name="t">the object ('tuple')</param>
		/// <returns>the convertes object as XmlNode</returns>
		public static XmlNode convertToXML(ExtendedXmlDocument xmldoc, Object t) 
		{
			XmlNode tupleNode = xmldoc.CreateElement("TupleSpace");
			if(Datatypes.isXMLTemplate(t))
			{
				tupleNode = stripXMLTuple((XMLTemplate)t, xmldoc);
			}
			else if(Datatypes.isXMLTuple(t))
			{
				tupleNode = stripXMLTuple((ITuple) t, xmldoc);
			} 
			else if(Datatypes.isObject(t)) 
			{
				tupleNode = buildSoapNode(xmldoc, t);
			} 
			else 
			{
				//logger.Debug("Unrecognized type found...");
			}
			return tupleNode;
		}

		/// <summary>
		/// converts an XmlField to an XmlNode to add to an existing XmlDocument
		/// </summary>
		/// <param name="xmldoc">the Xmldoc where the field created to</param>
		/// <param name="f">the field to create</param>
		/// <returns>the field as XmlNode</returns>
		public static XmlNode convertFieldToXML(ExtendedXmlDocument xmldoc, XMLField f)
		{
				XmlNode newElem = xmldoc.CreateElement("Field");
				((XmlElement)newElem).SetAttribute("type", f.Type.ToString());
				newElem.InnerText = f.FieldValue.ToString();
			return newElem;
		}
		/// <summary>
		/// converts an Xmldoc to an XmlNode to add to an existing XmlDocument as a field
		/// </summary>
		/// <param name="xmldoc">the Xmldoc where the field created to</param>
		/// <param name="s">the xmldoc to convert</param>
		/// <returns>the xmldoc in a field as XmlNode</returns>
		public static XmlNode convertFieldToXML(ExtendedXmlDocument xmldoc, ExtendedXmlDocument s)
		{
				XmlNode newElem = xmldoc.CreateElement("Field");
				((XmlElement)newElem).SetAttribute("type", "XMLDocument");
				newElem.InnerText = s.OuterXml;
				return newElem;
		}
		/// <summary>
		/// converts an IMatchable object as XmlField to an XmlNode to add to an existing XmlDocument
		/// </summary>
		/// <param name="xmldoc">the Xmldoc where the IMatchable object created to</param>
		/// <param name="m">the IMatchable object to create</param>
		/// <returns>the IMatchable object in a field as XmlNode</returns>
		public static XmlNode convertFieldToXML(ExtendedXmlDocument xmldoc, IMatchable m)
		{
			XmlNode newElem = buildSoapNode(xmldoc, m);
			((XmlElement) newElem).SetAttribute("type", "IMatchable");
			return newElem;
		}
		/// <summary>
		/// converts an IXMLMatchable object as XmlField to an XmlNode to add to an existing XmlDocument
		/// </summary>
		/// <param name="xmldoc">the Xmldoc where the IXMLMatchable object created to</param>
		/// <param name="m">the IXMLMatchable object to create</param>
		/// <returns>the IXMLMatchable object in a field as XmlNode</returns>
		public static XmlNode convertFieldToXML(ExtendedXmlDocument xmldoc, IXMLMatchable m)
		{
			XmlNode newElem = buildSoapNode(xmldoc, m);
			((XmlElement) newElem).SetAttribute("type", "IXMLMatchable");
			return newElem;
		}
		
		
		/// <summary>
		/// gets the fields in the tuples in the xmldoc
		/// </summary>
		/// <param name="xmldoc">the Xmldoc where to find the fields</param>
		/// <returns>the fields as XmlNodes in a XmlNodeList</returns>
		// vom Tuple im TupleSpace
		public static XmlNodeList getFields(ExtendedXmlDocument xmldoc)
		{
			string exprstr = "TupleSpace/Tuple/Field";
			XmlNodeList nodelist = xmldoc.SelectNodes(exprstr);
			return nodelist;
		}
		/// <summary>
		/// gets the type- attribut value of the XmlNode
		/// </summary>
		/// <param name="node">the XmlNode where to get the value</param>
		/// <returns>the type attribut value</returns>
		public static object getFieldType(XmlNode node)
		{
			object fieldtype = ((XmlElement) node).GetAttribute("type");
			return fieldtype;
		}
		/// <summary>
		/// gets the field value of the XmlNode
		/// </summary>
		/// <param name="node">the XmlNode where to get the value</param>
		/// <returns>the value of the field</returns>
		public static object getFieldvalue(XmlNode node)
		{
			object fieldvalue = node.InnerText;
			return fieldvalue;
		}

		/// <summary>
		/// Serializes an object to an Xml representation (in soap)
		/// </summary>
		/// <param name="xmldoc">the Xmldoc for append the node</param>
		/// <param name="obj">the object to build for</param>
		/// <returns>the object in soap as XmlNode</returns>
		public static XmlNode buildSoapNode(ExtendedXmlDocument xmldoc, Object obj)
		{
			if(obj.GetType().ToString().Equals("System.MarshalByRefObject")) 
			{
				IMatchable o = obj as IMatchable;
				XmlNode newNode = xmldoc.CreateElement("MarshalByRefObject");
				newNode.InnerXml = o.GetType().ToString() + " denkste!";
				return newNode;
			}
			try 
			{
				SoapFormatter soap = new SoapFormatter();
				MemoryStream mstream = new MemoryStream();
				// serialize to xml
				soap.Serialize(mstream, obj);
				// convert bytes back to characters
				byte[] buffer;
				buffer = mstream.GetBuffer();
				System.Text.Encoding enc = new System.Text.UTF8Encoding();
				String xmlstring = enc.GetString(buffer, 0, buffer.Length);
				//Console.WriteLine(xmlstring);
				// create a DOM - representation from xmlstring
				ExtendedXmlDocument newSoap = new ExtendedXmlDocument();
				newSoap.LoadXml(xmlstring);
				// create new tuplenode and import soapnode
				XmlNode newNode = xmldoc.CreateElement("Field"); // soap obj ist ein feld, kein tuple mehr
				((XmlElement) newNode).SetAttribute("type", "Soap");
				XmlNode soapNode = xmldoc.ImportNode(newSoap.DocumentElement, true);
				newNode.AppendChild(soapNode);
				return newNode;
			} 
			catch(Exception ex) 
			{
				XmlNode newNode = xmldoc.CreateElement("Error");
				newNode.InnerXml = ex.Message + " ________ " + obj.GetType().ToString();
				return newNode;
			}			
		}
		/// <summary>
		/// builds a field with a Xmldoc
		/// </summary>
		/// <param name="xmldoc">the Xmldoc for append the converted node</param>
		/// <param name="exdoc">the Xmldoc to convert to</param>
		/// <returns>the Xmldoc in a field as XmlNode</returns>
		public static XmlNode buildXmlDocumentNode(ExtendedXmlDocument xmldoc, ExtendedXmlDocument exdoc) 
		{
			XmlNode newNode = xmldoc.CreateElement("Field");
			((XmlElement) newNode).SetAttribute("type", "XmlDocument");
			XmlNode node = xmldoc.ImportNode(exdoc.DocumentElement, true);
			newNode.AppendChild(node);
			return newNode;
		}

		/// <summary>
		/// strips an XmlTuple to a node
		/// </summary>
		/// <param name="tuple">the ITuple to strip to</param>
		/// <param name="xmldoc">the Xmldoc for append the stripped ITuple</param>
		/// <returns>the ITuple as XmlNode</returns>
		public static XmlNode stripXMLTuple(ITuple tuple, ExtendedXmlDocument xmldoc) 
		{
			ExtendedXmlDocument mydoc = ((XMLTuple)tuple).getXmlDocument();
			XmlNode newNode = xmldoc.CreateElement("Tuple");
			((XmlElement) newNode).SetAttribute("type", mydoc.DocumentElement.GetAttribute("type"));
			((XmlElement) newNode).SetAttribute("tuplecount", mydoc.DocumentElement.GetAttribute("tuplecount"));
			((XmlElement) newNode).SetAttribute("fieldcount", mydoc.DocumentElement.GetAttribute("fieldcount"));

			foreach(XmlNode n in mydoc.DocumentElement.ChildNodes) 
			{
				XmlNode node = xmldoc.ImportNode(n, true);
				newNode.AppendChild(node);
			}
			return newNode;
		}
		/// <summary>
		/// strips an XMLTemplate to a node
		/// </summary>
		/// <param name="tuple">the XMLTemplate to strip to</param>
		/// <param name="xmldoc">the Xmldoc for append the stripped XMLTemplate</param>
		/// <returns>the XMLTemplate as XmlNode</returns>
		public static XmlNode stripXMLTuple(XMLTemplate tuple, ExtendedXmlDocument xmldoc) 
		{
			ExtendedXmlDocument mydoc = ((XMLTemplate)tuple).getXmlDocument();
			XmlNode newNode = xmldoc.CreateElement("Template");
			((XmlElement) newNode).SetAttribute("type", tuple.GetType().ToString());
			((XmlElement) newNode).SetAttribute("tuplecount", mydoc.DocumentElement.GetAttribute("tuplecount"));
			((XmlElement) newNode).SetAttribute("fieldcount", mydoc.DocumentElement.GetAttribute("fieldcount"));

			foreach(XmlNode n in mydoc.DocumentElement.ChildNodes) 
			{
				XmlNode node = xmldoc.ImportNode(n, true);
				newNode.AppendChild(node);
			}
			return newNode;
		}
	}
}
