using System;
using System.Xml;
using System.Runtime.Remoting;
using System.Runtime.Remoting.Channels;
using System.Runtime.Remoting.Channels.Http;
using System.Collections;
using System.Threading;
using Interfaces;
using System.IO;
//using log4net;
//using log4net.Config;
using Helpers;
using System.Net;

namespace XMLTupleSpaceShared 
{
	/// <summary>
	/// Provides an interface to the tuplespace-server
	/// </summary>
	/// 
	public class XMLTupleSpace : MarshalByRefObject
	{
//		private static readonly ILog logger = LogManager.GetLogger(typeof(XMLTupleSpace));
//		private FileInfo fnfo = new FileInfo("..\\..\\logconfig.xml");
		/// <summary>
		/// the tuplespaces' name
		/// </summary>
		private String myName = "";
		/// <summary>
		/// the server this tuplespace is on
		/// </summary>
		private String server = "";
		/// <summary>
		/// The TupleSpaceServer this TupleSpace is talking to
		/// </summary>
		private XMLServer tsServer;
		//private readonly int port = 65100; // besser aus einer config-datei
		//private readonly string endpoint = "XMLTupleSpace";

		/// <summary>
		/// Constructor that connects to server to aquire the server-object
		/// </summary>
		/// <param name="str">the tuplespace's name</param>
		/// <param name="servername">the server's url</param>
		public XMLTupleSpace(String str, String servername)
		{
//			DOMConfigurator.Configure(fnfo);
			this.myName = str;
			this.server = servername;
			tsServer = (XMLServer) Activator.GetObject(typeof(XMLServer), servername);
		}

		/// <summary>
		/// Constructor takes a reference to a tuplespaceserver
		/// </summary>
		/// <param name="str">the tuplespace's name</param>
		/// <param name="servername">the server's name</param>
		/// <param name="server">a reference to the server that created this tuplespace</param>
		public XMLTupleSpace(String str, String servername, XMLServer server) 
		{
//			DOMConfigurator.Configure(fnfo);
			this.myName = str;
			this.server = servername;
			tsServer = server;
		}

		/// <summary>
		/// Constructor does not care about server's position. 
		/// </summary>
		/// <param name="tsName">the tuplespace's name</param>
		/// <param name="server">a reference to the server this tuplespace has been created on</param>
		public XMLTupleSpace(String tsName, XMLServer server) 
		{
//			DOMConfigurator.Configure(fnfo);
			this.myName = tsName;
			tsServer = server;
		}

//		/// <summary>
//		/// returns the port the referenced server ist listening on
//		/// </summary>
//		/// <returns>the port used to communicate with the server</returns>
//		public int getPort() 
//		{
//			try 
//			{
//				/*UriBuilder myURI = new UriBuilder("http", Dns.GetHostByName(Dns.GetHostName()).AddressList[0].ToString());
//				myURI.Port = port;
//				*/
//				return port;
//			}
//			catch(Exception e) 
//			{
////				logger.Error("Could not parse server's port", e);
//				return 0;
//			}
//		}

		/// <summary>
		/// corresponds to the <c>out</c> of linda-systems
		/// </summary>
		/// <param name="t">the tuple to be written to the tuplespace</param>
		public void output(ITuple t) 
		{
			try 
			{
				tsServer.addToTs(myName, t);			
			} 
			catch (Exception e) 
			{
//				logger.Error("Could not write to tuplespace : " + e.Message);
			}
		}

		/// <summary>
		/// corresponds to the <c>'in'</c> in linda-systems
		/// </summary>
		/// <param name="t">the tuple to consume</param>
		/// <returns>the tuple if any was found</returns>
		public ITuple input(ITuple t) 
		{
			ITuple temp = input(new DeepTemplate(t)); 
			return temp;
		}
		
		/// <summary>
		/// corresponds to the <c>'in'</c> in linda-systems
		/// </summary>
		/// <param name="t">the template for matching</param>
		/// <returns>the tuple if any matching the template was found</returns>
		public ITuple input(XMLTemplate template) 
		{
			Boolean found = false;
			ITuple returnedTuple = null;
			while(!found) 
			{
				returnedTuple = tsServer.syncRemove(myName, template);				
				if(returnedTuple == null)	
				{ 
					Thread.Sleep(3000); }
				else		{ found = true;	}
			}
			return returnedTuple;
		}
//		/// <summary>
//		/// evaluates the statement(s) in a tuple and 'outs' the tuple to tuplespace
//		/// </summary>
//		/// <returns></returns>
//		public ITuple eval() { return new XMLTuple(); }
		/// <summary>
		/// reads a tuple without consuming it
		/// </summary>
		/// <returns>the tuple that's been found</returns>
		public ITuple read(XMLTemplate template) { 
			Boolean found = false;
			ITuple returnedTuple = null;
			while(!found) 
			{
				returnedTuple = tsServer.syncReadFromTS(myName, template);				
				if(returnedTuple == null)	
				{ 
					Thread.Sleep(3000); }
				else		{ found = true;	}
			}
			
			return returnedTuple;
		}
	}
}
